@@ -19,7 +19,7 @@
 #error "VDR-1.5.8 API version or greater is required!"
 #endif
 
-static const char VERSION[]       = "1.2.3";
+static const char VERSION[]       = "1.2.4";
 static const char DESCRIPTION[]   = trNOOP("DVB Signal Information Monitor (OSD)");
 static const char MAINMENUENTRY[] = trNOOP("Signal Information");
 
@@ -109,6 +109,7 @@ bool cPluginFemon::SetupParse(const char *Name, const char *Value)
 {
   // Parse your own setup parameters and store their values.
   if      (!strcasecmp(Name, "HideMenu"))       femonConfig.hidemenu       = atoi(Value);
+  else if (!strcasecmp(Name, "UseSingleArea"))  femonConfig.usesinglearea  = atoi(Value);
   else if (!strcasecmp(Name, "DisplayMode"))    femonConfig.displaymode    = atoi(Value);
   else if (!strcasecmp(Name, "Position"))       femonConfig.position       = atoi(Value);
   else if (!strcasecmp(Name, "OSDHeight"))      femonConfig.osdheight      = atoi(Value);
@@ -133,18 +134,20 @@ bool cPluginFemon::SetupParse(const char *Name, const char *Value)
 
 bool cPluginFemon::Service(const char *Id, void *Data)
 {
-  if ((strcmp(Id,"FemonService-v1.0") == 0) && Data) {
-     FemonService_v1_0 *data = (FemonService_v1_0*)Data;
-     int ndx = cDevice::ActualDevice()->CardIndex();
-     data->fe_name = getFrontendName(ndx);
-     data->fe_status = getFrontendStatus(ndx);
-     data->fe_snr = getSNR(ndx);
-     data->fe_signal = getSignal(ndx);
-     data->fe_ber = getBER(ndx);
-     data->fe_unc = getUNC(ndx);
-     data->video_bitrate = cFemonOsd::Instance() ? cFemonOsd::Instance()->GetVideoBitrate() : 0.0;
-     data->audio_bitrate = cFemonOsd::Instance() ? cFemonOsd::Instance()->GetAudioBitrate() : 0.0;
-     data->dolby_bitrate = cFemonOsd::Instance() ? cFemonOsd::Instance()->GetDolbyBitrate() : 0.0;
+  if (strcmp(Id,"FemonService-v1.0") == 0) {
+     if (Data) {
+        FemonService_v1_0 *data = (FemonService_v1_0*)Data;
+        int ndx = cDevice::ActualDevice()->CardIndex();
+        data->fe_name = getFrontendName(ndx);
+        data->fe_status = getFrontendStatus(ndx);
+        data->fe_snr = getSNR(ndx);
+        data->fe_signal = getSignal(ndx);
+        data->fe_ber = getBER(ndx);
+        data->fe_unc = getUNC(ndx);
+        data->video_bitrate = cFemonOsd::Instance() ? cFemonOsd::Instance()->GetVideoBitrate() : 0.0;
+        data->audio_bitrate = cFemonOsd::Instance() ? cFemonOsd::Instance()->GetAudioBitrate() : 0.0;
+        data->dolby_bitrate = cFemonOsd::Instance() ? cFemonOsd::Instance()->GetDolbyBitrate() : 0.0;
+        }
      return true;
      }
 
@@ -306,7 +309,10 @@ void cMenuFemonSetup::Setup(void)
   help.Clear();
 
   Add(new cMenuEditBoolItem(tr("Hide main menu entry"), &data.hidemenu));
-  help.Append(tr("Define whether the main manu entry is hidden."));
+  help.Append(tr("Define whether the main menu entry is hidden."));
+
+  Add(new cMenuEditBoolItem(tr("Use single area (8bpp)"), &data.usesinglearea));
+  help.Append(tr("Define whether a single 8bpp OSD area is preferred.\n\nRequired by Truetype fonts and anti-aliasing."));
 
   Add(new cMenuEditStraItem(tr("Default display mode"), &data.displaymode, eFemonModeMaxNumber, dispmodes));
   help.Append(tr("Define the default display mode at startup."));
@@ -366,6 +372,7 @@ void cMenuFemonSetup::Store(void)
   Dprintf("%s()\n", __PRETTY_FUNCTION__);
   femonConfig = data;
   SetupStore("HideMenu",       femonConfig.hidemenu);
+  SetupStore("UseSingleArea",  femonConfig.usesinglearea);
   SetupStore("DisplayMode",    femonConfig.displaymode);
   SetupStore("Skin",           femonConfig.skin);
   SetupStore("Theme",          femonConfig.theme);
