@@ -11,7 +11,6 @@
 
 #ifdef FREEBOXTV_EPG
 #include <libxml++/libxml++.h>
-#include <stdio.h>
 #endif
 
 // VDR includes
@@ -30,6 +29,7 @@
 
 static const char *VERSION        = "0.0.3";
 static const char *DESCRIPTION    = "Watch FreeboxTv with VDR";
+static const char *plugName       = "FreeboxTV Plugin";
 
 using namespace FreeboxTv;
 using FreeboxTv::cFreeboxClient;
@@ -41,10 +41,6 @@ extern char *url;
 
 extern UsageEnvironment *env;
 unsigned short ClientPortNum;   // Port client  utiliser : 0 = random value
-#ifdef FREEBOXTV_EPG
-int UpdateEPG; // Periode de rechragement EPG
-int RenameFile;
-#endif
 extern cRingBufferLinear *TSDataBuffer;
 
 extern cMutex fbmutex;
@@ -88,7 +84,7 @@ bool cFreeboxChannel::Parse(const char *s)
   }
 
   free (fbtid);
-  delete[] fbChId;
+  delete fbChId;
   return channelID.Valid();
 }
 
@@ -199,7 +195,7 @@ bool cFreeboxDevice::GetTSPacket(uchar *&Data)
 
 // Comme la lecture n'est pas bloquante, on temporise pour viter un poll trop agressif de VDR
   if (!Data)
-    cCondWait::SleepMs(1000);
+    cCondWait::SleepMs(10);
 
   return true;
 }
@@ -267,10 +263,6 @@ bool cFreeboxDevice::SwitchChannel (cFreeboxChannel *Channel, bool LiveView)
 class cMenuSetupFreeboxTv : public cMenuSetupPage {
 private:
   int newClientPortNum;
-#ifdef FREEBOXTV_EPG
-  int newUpdateEPG;
-  int newRenameFile;
-#endif
 protected:
   virtual void Store(void);
 public:
@@ -280,24 +272,12 @@ public:
 cMenuSetupFreeboxTv::cMenuSetupFreeboxTv(void)
 {
   newClientPortNum = ClientPortNum;
-#ifdef FREEBOXTV_EPG
-  newUpdateEPG = UpdateEPG;
-  newRenameFile = RenameFile;
-#endif
   Add(new cMenuEditIntItem( tr("Client Port Number (0 = random) "), &newClientPortNum));
-#ifdef FREEBOXTV_EPG
-  Add(new cMenuEditIntItem( tr("Period of loading EPG (hours) "), &newUpdateEPG));
-  Add(new cMenuEditBoolItem( tr("Rename xmltv.freebox to xmltv.freebox.save " ), &newRenameFile));
-#endif
 }
 
 void cMenuSetupFreeboxTv::Store(void)
 {
   SetupStore("ClientPortNum", ClientPortNum = newClientPortNum);
-#ifdef FREEBOXTV_EPG
-  SetupStore("UpdateEPG", UpdateEPG = newUpdateEPG);
-  SetupStore("RenameFile", RenameFile = newRenameFile);
-#endif
 }
 
 // --- cPluginFreeboxTv-------------------------------------------------------
@@ -362,18 +342,12 @@ bool cPluginFreeboxTv::Initialize(void)
   FreeboxEpg->SetLastEpgUpdate(0);
 
   // Read and parse XMLTV file
-  int result;
-  FILE *f = fopen(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"), "r");
-  if (f != NULL) {
-     if (FreeboxEpg->Parse(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"))) {
-        FreeboxEpg->SetLastEpgUpdate();
-        if (RenameFile) {
-           result = rename((AddDirectory(ConfigDirectory(Name()), "xmltv.freebox")),(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox.save")));
-           if ( result != 0 )
-              esyslog("freeboxtv: ERROR: Rename file");
-        }
-     }
-   }
+  if (FreeboxEpg->Parse(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"))) {
+    FreeboxEpg->SetLastEpgUpdate();
+  }
+  else {
+    esyslog("ERROR: error reading xmltv.freebox file");
+  }
 #endif
 
   // No or bad EPG is not a fatal error
@@ -385,22 +359,10 @@ void cPluginFreeboxTv::Housekeeping(void)
   // Perform any cleanup or other regular tasks.
 
 #ifdef FREEBOXTV_EPG
-   int result;
-   FILE *f = fopen(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"), "r");
-   // Mise  jour des donnes EPG de VDR si cela n'a pas t fait depuis au moins dfinit par UpdateEPG
-   if (UpdateEPG > 0) {
-      if ((int)(time(NULL) - FreeboxEpg->GetLastEpgUpdate()) > (UpdateEPG * 60 * 60))
-         if (f != NULL) {
-            if (FreeboxEpg->Parse(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"))) {
-               FreeboxEpg->SetLastEpgUpdate();
-               if (RenameFile) {
-                  result = rename((AddDirectory(ConfigDirectory(Name()), "xmltv.freebox")),(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox.save")));
-                  if ( result != 0 )
-                     esyslog("freeboxtv: ERROR: Rename file");
-               }
-            }
-         }
-   }
+  // Mise  jour des donnes EPG de VDR si cela n'a pas t fait depuis au moins 6 heures
+  if ((int)(time(NULL) - FreeboxEpg->GetLastEpgUpdate()) > 21600)
+      if (FreeboxEpg->Parse(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox")))
+        FreeboxEpg->SetLastEpgUpdate();
 #endif
 }
 
@@ -415,12 +377,6 @@ bool cPluginFreeboxTv::SetupParse(const char *Name, const char *Value)
   // Parse your own setup parameters and store their values.
   if (!strcasecmp(Name, "ClientPortNum"))
     ClientPortNum = atoi(Value);
-#ifdef FREEBOXTV_EPG
-  else if (!strcasecmp(Name, "UpdateEPG"))
-     UpdateEPG = atoi(Value);
-  else if (!strcasecmp(Name, "RenameFile"))
-     RenameFile = atoi(Value);
-#endif
   else
     return false;
 
@@ -430,48 +386,12 @@ bool cPluginFreeboxTv::SetupParse(const char *Name, const char *Value)
 const char **cPluginFreeboxTv::SVDRPHelpPages(void)
 {
   // Return help text for SVDRP commands this plugin implements
-#ifdef FREEBOXTV_EPG
-  static const char *HelpPages[] = {
-    "LOAD"
-    "    Loading xmltv.freeboxtv.",
-    NULL
-    };
-    return HelpPages;
-#else
-    return NULL;
-#endif
+  return NULL;
 }
 
 cString cPluginFreeboxTv::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
 {
   // Process SVDRP commands this plugin implements
-#ifdef FREEBOXTV_EPG
-  int result;
-  if(!strcasecmp(Command,"LOAD")) {
-    FILE *f = fopen(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"), "r");
-    if (f != NULL) {
-      if (FreeboxEpg->Parse(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox"))) {
-        FreeboxEpg->SetLastEpgUpdate();
-        if (RenameFile) {
-          result = rename((AddDirectory(ConfigDirectory(Name()), "xmltv.freebox")),(AddDirectory(ConfigDirectory(Name()), "xmltv.freebox.save")));
-          if ( result != 0 ) {
-            ReplyCode=550;
-            return "ERROR: Rename file";
-          }
-        }
-        return "Loading EPG successfully";
-      }
-    }
-    else {
-      ReplyCode=550;
-      return "ERROR: File not found";
-    }
-  }
-  else {
-    ReplyCode=550;
-    return "Loading EPG failed";
-  }
-#endif
   return NULL;
 }
 
