@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: config.h,v 1.18 2008/01/04 23:36:37 ajhseppa Exp $
+ * $Id: config.h,v 1.20 2008/02/01 21:54:24 rahrenbe Exp $
  */
 
 #ifndef __IPTV_CONFIG_H
@@ -21,7 +21,6 @@ private:
   unsigned int extProtocolBasePort;
   unsigned int useBytes;
   unsigned int sectionFiltering;
-  unsigned int sidScanning;
   int disabledFilters[SECTION_FILTER_TABLE_SIZE];
   char configDirectory[255];
 
@@ -33,7 +32,6 @@ public:
   unsigned int GetExtProtocolBasePort(void) const { return extProtocolBasePort; }
   unsigned int GetUseBytes(void) const { return useBytes; }
   unsigned int GetSectionFiltering(void) const { return sectionFiltering; }
-  unsigned int GetSidScanning(void) const { return sidScanning; }
   const char *GetConfigDirectory(void) const { return configDirectory; }
   unsigned int GetDisabledFiltersCount(void) const;
   int GetDisabledFilters(unsigned int Index) const;
@@ -42,7 +40,6 @@ public:
   void SetExtProtocolBasePort(unsigned int PortNumber) { extProtocolBasePort = PortNumber; }
   void SetUseBytes(unsigned int On) { useBytes = On; }
   void SetSectionFiltering(unsigned int On) { sectionFiltering = On; }
-  void SetSidScanning(unsigned int On) { sidScanning = On; }
   void SetDisabledFilters(unsigned int Index, int Number);
   void SetConfigDirectory(const char *directoryP);
 };
