@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: device.c,v 1.78 2008/01/19 21:08:02 ajhseppa Exp $
+ * $Id: device.c,v 1.84 2008/02/17 19:18:47 rahrenbe Exp $
  */
 
 #include "config.h"
@@ -17,8 +17,11 @@ unsigned int cIptvDevice::deviceCount = 0;
 
 cIptvDevice::cIptvDevice(unsigned int Index)
 : deviceIndex(Index),
+  dvrFd(-1),
   isPacketDelivered(false),
   isOpenDvr(false),
+  sidScanEnabled(false),
+  pidScanEnabled(false),
   mutex()
 {
   debug("cIptvDevice::cIptvDevice(%d)\n", deviceIndex);
@@ -32,6 +35,7 @@ cIptvDevice::cIptvDevice(unsigned int Index)
   pFileProtocol = new cIptvProtocolFile();
   pExtProtocol = new cIptvProtocolExt();
   pIptvStreamer = new cIptvStreamer(tsBuffer, &mutex);
+  pPidScanner = new cPidScanner;
   // Initialize filter pointers
   memset(secfilters, '\0', sizeof(secfilters));
   // Start section handler for iptv device
@@ -40,17 +44,31 @@ cIptvDevice::cIptvDevice(unsigned int Index)
   pSidScanner = new cSidScanner;
   if (pSidScanner)
      AttachFilter(pSidScanner);
+  // Check if dvr fifo exists
+  struct stat sb;
+  cString filename = cString::sprintf(IPTV_DVR_FILENAME, deviceIndex);
+  stat(filename, &sb);
+  if (S_ISFIFO(sb.st_mode)) {
+     dvrFd = open(filename, O_WRONLY | O_NONBLOCK);
+     if (dvrFd >= 0)
+        dsyslog("IPTV device %d redirecting input stream to '%s'", deviceIndex, *filename);
+     }
 }
 
 cIptvDevice::~cIptvDevice()
 {
   debug("cIptvDevice::~cIptvDevice(%d)\n", deviceIndex);
+#if defined(APIVERSNUM) && APIVERSNUM >= 10515
+  // Stop section handler of iptv device
+  StopSectionHandler();
+#endif
   DELETE_POINTER(pIptvStreamer);
   DELETE_POINTER(pUdpProtocol);
   DELETE_POINTER(pHttpProtocol);
   DELETE_POINTER(pFileProtocol);
   DELETE_POINTER(pExtProtocol);
   DELETE_POINTER(tsBuffer);
+  DELETE_POINTER(pPidScanner);
   // Detach and destroy sid filter
   if (pSidScanner) {
      Detach(pSidScanner);
@@ -59,6 +77,12 @@ cIptvDevice::~cIptvDevice()
   // Destroy all filters
   for (int i = 0; i < eMaxSecFilterCount; ++i)
       DeleteFilter(i);
+  // Close dvr fifo
+  if (dvrFd >= 0) {
+     int fd = dvrFd;
+     dvrFd = -1;
+     close(fd);
+     }
 }
 
 bool cIptvDevice::Initialize(unsigned int DeviceCount)
@@ -155,13 +179,13 @@ cString cIptvDevice::GetInformation(unsigned int Page)
   return info;
 }
 
-cString cIptvDevice::GetChannelSettings(const char *IptvParam, int *Parameter, cIptvProtocolIf* *Protocol)
+cString cIptvDevice::GetChannelSettings(const char *IptvParam, int *Parameter, int *SidScan, int *PidScan, cIptvProtocolIf* *Protocol)
 {
   debug("cIptvDevice::GetChannelSettings(%d)\n", deviceIndex);
   char *tag = NULL;
   char *proto = NULL;
   char *loc = NULL;
-  if (sscanf(IptvParam, "%a[^|]|%a[^|]|%a[^|]|%u", &tag, &proto, &loc, Parameter) == 4) {
+  if (sscanf(IptvParam, "%a[^|]|S%dP%d|%a[^|]|%a[^|]|%u", &tag, SidScan, PidScan, &proto, &loc, Parameter) == 6) {
      cString tagstr(tag, true);
      cString protostr(proto, true);
      cString locstr(loc, true);
@@ -182,6 +206,51 @@ cString cIptvDevice::GetChannelSettings(const char *IptvParam, int *Parameter, c
         return locstr;
         }
      }
+  else if (sscanf(IptvParam, "%a[^|]|P%dS%d|%a[^|]|%a[^|]|%u", &tag, PidScan, SidScan, &proto, &loc, Parameter) == 6) {
+     cString tagstr(tag, true);
+     cString protostr(proto, true);
+     cString locstr(loc, true);
+     // check if IPTV tag
+     if (strncasecmp(*tagstr, "IPTV", 4) == 0) {
+        // check if protocol is supported and update the pointer
+        if (strncasecmp(*protostr, "UDP", 3) == 0)
+           *Protocol = pUdpProtocol;
+        else if (strncasecmp(*protostr, "HTTP", 4) == 0)
+           *Protocol = pHttpProtocol;
+        else if (strncasecmp(*protostr, "FILE", 4) == 0)
+           *Protocol = pFileProtocol;
+        else if (strncasecmp(*protostr, "EXT", 3) == 0)
+           *Protocol = pExtProtocol;
+        else
+           return NULL;
+        // return location
+        return locstr;
+        }
+     }
+  // compatibility mode for old channels.conf format
+  else if (sscanf(IptvParam, "%a[^|]|%a[^|]|%a[^|]|%u", &tag, &proto, &loc, Parameter) == 4) {
+     cString tagstr(tag, true);
+     cString protostr(proto, true);
+     cString locstr(loc, true);
+     *SidScan = 0;
+     *PidScan = 0;
+     // check if IPTV tag
+     if (strncasecmp(*tagstr, "IPTV", 4) == 0) {
+        // check if protocol is supported and update the pointer
+        if (strncasecmp(*protostr, "UDP", 3) == 0)
+           *Protocol = pUdpProtocol;
+        else if (strncasecmp(*protostr, "HTTP", 4) == 0)
+           *Protocol = pHttpProtocol;
+        else if (strncasecmp(*protostr, "FILE", 4) == 0)
+           *Protocol = pFileProtocol;
+        else if (strncasecmp(*protostr, "EXT", 3) == 0)
+           *Protocol = pExtProtocol;
+        else
+           return NULL;
+        // return location
+        return locstr;
+        }
+     }
   return NULL;
 }
 
@@ -216,21 +285,30 @@ bool cIptvDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *N
   return result;
 }
 
+int cIptvDevice::NumProvidedSystems(void) const
+{
+  return 1;
+}
+
 bool cIptvDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
 {
-  int parameter;
+  int parameter, sidscan, pidscan;
   cString location;
   cIptvProtocolIf *protocol;
 
   debug("cIptvDevice::SetChannelDevice(%d)\n", deviceIndex);
-  location = GetChannelSettings(Channel->PluginParam(), &parameter, &protocol);
+  location = GetChannelSettings(Channel->PluginParam(), &parameter, &sidscan, &pidscan, &protocol);
   if (isempty(location)) {
      error("ERROR: Unrecognized IPTV channel settings: %s", Channel->PluginParam());
      return false;
      }
+  sidScanEnabled = sidscan ? true : false;
+  pidScanEnabled = pidscan ? true : false;
   pIptvStreamer->Set(location, parameter, deviceIndex, protocol);
-  if (pSidScanner && IptvConfig.GetSectionFiltering() && IptvConfig.GetSidScanning())
+  if (sidScanEnabled && pSidScanner && IptvConfig.GetSectionFiltering())
      pSidScanner->SetChannel(Channel);
+  if (pidScanEnabled && pPidScanner)
+     pPidScanner->SetChannel(Channel);
   return true;
 }
 
@@ -309,7 +387,7 @@ bool cIptvDevice::OpenDvr(void)
   mutex.Unlock();
   ResetBuffering();
   pIptvStreamer->Open();
-  if (pSidScanner && IptvConfig.GetSectionFiltering() && IptvConfig.GetSidScanning())
+  if (sidScanEnabled && pSidScanner && IptvConfig.GetSectionFiltering())
      pSidScanner->SetStatus(true);
   isOpenDvr = true;
   return true;
@@ -318,10 +396,10 @@ bool cIptvDevice::OpenDvr(void)
 void cIptvDevice::CloseDvr(void)
 {
   debug("cIptvDevice::CloseDvr(%d)\n", deviceIndex);
-  if (pSidScanner && IptvConfig.GetSectionFiltering() && IptvConfig.GetSidScanning())
+  if (pidScanEnabled && pSidScanner && IptvConfig.GetSectionFiltering())
      pSidScanner->SetStatus(false);
   if (pIptvStreamer)
-    pIptvStreamer->Close();
+     pIptvStreamer->Close();
   isOpenDvr = false;
 }
 
@@ -376,8 +454,14 @@ bool cIptvDevice::GetTSPacket(uchar *&Data)
            }
         isPacketDelivered = true;
         Data = p;
-	// Update pid statistics 
-	AddPidStatistic(ts_pid(p), payload(p));
+        // Update pid statistics 
+        AddPidStatistic(ts_pid(p), payload(p));
+        // Send data also to dvr fifo
+        if (dvrFd >= 0)
+           write(dvrFd, p, TS_SIZE);
+        // Analyze incomplete streams with built-in pid analyzer
+        if (pidScanEnabled && pPidScanner)
+            pPidScanner->Process(p);
         // Run the data through all filters
         for (unsigned int i = 0; i < eMaxSecFilterCount; ++i) {
             if (secfilters[i])
