@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: iptv.c,v 1.33 2008/01/19 20:24:10 rahrenbe Exp $
+ * $Id: iptv.c,v 1.36 2008/02/01 21:54:24 rahrenbe Exp $
  */
 
 #include <getopt.h>
@@ -22,7 +22,7 @@
 #error "You must apply the closefilter patch for VDR!"
 #endif
 
-static const char VERSION[]     = "0.0.6";
+static const char VERSION[]     = "0.0.7";
 static const char DESCRIPTION[] = trNOOP("Experience the IPTV");
 
 class cPluginIptv : public cPlugin {
@@ -79,20 +79,20 @@ bool cPluginIptv::ProcessArgs(int argc, char *argv[])
   debug("cPluginIptv::ProcessArgs()\n");
   // Implement command line argument processing here if applicable.
   static const struct option long_options[] = {
-       { "devices", required_argument, NULL, 'd' },
-       { NULL }
-     };
+    { "devices", required_argument, NULL, 'd' },
+    { NULL }
+    };
 
   int c;
   while ((c = getopt_long(argc, argv, "d:", long_options, NULL)) != -1) {
-        switch (c) {
-          case 'd':
-               deviceCount = atoi(optarg);
-               break;
-          default:
-               return false;
-          }
-        }
+    switch (c) {
+      case 'd':
+           deviceCount = atoi(optarg);
+           break;
+      default:
+           return false;
+      }
+    }
   return true;
 }
 
@@ -167,14 +167,14 @@ int cPluginIptv::ParseFilters(const char *Value, int *Filters)
   char buffer[256];
   int n = 0;
   while (Value && *Value && (n < SECTION_FILTER_TABLE_SIZE)) {
-        strn0cpy(buffer, Value, sizeof(buffer));
-        int i = atoi(buffer);
-        //debug("cPluginIptv::ParseFilters(): Filters[%d]=%d\n", n, i);
-        if (i >= 0)
-           Filters[n++] = i;
-        if ((Value = strchr(Value, ' ')) != NULL)
-           Value++;
-        }
+    strn0cpy(buffer, Value, sizeof(buffer));
+    int i = atoi(buffer);
+    //debug("cPluginIptv::ParseFilters(): Filters[%d]=%d\n", n, i);
+    if (i >= 0)
+       Filters[n++] = i;
+    if ((Value = strchr(Value, ' ')) != NULL)
+       Value++;
+    }
   return n;
 }
 
@@ -190,8 +190,6 @@ bool cPluginIptv::SetupParse(const char *Name, const char *Value)
      IptvConfig.SetExtProtocolBasePort(atoi(Value));
   else if (!strcasecmp(Name, "SectionFiltering"))
      IptvConfig.SetSectionFiltering(atoi(Value));
-  else if (!strcasecmp(Name, "SidScanning"))
-     IptvConfig.SetSidScanning(atoi(Value));
   else if (!strcasecmp(Name, "DisabledFilters")) {
      int DisabledFilters[SECTION_FILTER_TABLE_SIZE];
      for (unsigned int i = 0; i < ARRAY_SIZE(DisabledFilters); ++i)
