@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: protocolhttp.c,v 1.22 2008/01/04 23:36:37 ajhseppa Exp $
+ * $Id: protocolhttp.c,v 1.24 2008/02/17 19:18:47 rahrenbe Exp $
  */
 
 #include <sys/types.h>
@@ -88,17 +88,14 @@ bool cIptvProtocolHttp::Connect(void)
         }
 
      // Formulate and send HTTP request
-     char buffer[256];
-     memset(buffer, '\0', sizeof(buffer));
-     snprintf(buffer, sizeof(buffer),
-	      "GET %s HTTP/1.1\r\n"
-	      "Host: %s\r\n"
-	      "User-Agent: vdr-iptv\r\n"
-	      "Range: bytes=0-\r\n"
-	      "Connection: Close\r\n"
-	      "\r\n", streamPath, streamAddr);
+     cString buffer = cString::sprintf("GET %s HTTP/1.1\r\n"
+                                       "Host: %s\r\n"
+                                       "User-Agent: vdr-iptv\r\n"
+                                       "Range: bytes=0-\r\n"
+                                       "Connection: Close\r\n"
+                                       "\r\n", streamPath, streamAddr);
 
-     //debug("Sending http request: %s\n", buffer);
+     debug("Sending http request: %s\n", *buffer);
      err = send(socketDesc, buffer, strlen(buffer), 0);
      ERROR_IF_FUNC(err < 0, "send()", CloseSocket(), return false);
 
@@ -119,10 +116,10 @@ bool cIptvProtocolHttp::Disconnect(void)
   debug("cIptvProtocolHttp::Disconnect()\n");
   // Check that stream address is valid
   if (isActive) {
-      // Close the socket
-      CloseSocket();
-      // Update active flag
-      isActive = false;
+     // Close the socket
+     CloseSocket();
+     // Update active flag
+     isActive = false;
      }
   return true;
 }
@@ -228,22 +225,22 @@ bool cIptvProtocolHttp::Set(const char* Location, const int Parameter, const int
 {
   debug("cIptvProtocolHttp::Set(): Location=%s Parameter=%d Index=%d\n", Location, Parameter, Index);
   if (!isempty(Location)) {
-    // Disconnect the current socket
-    Disconnect();
-    // Update stream address, path and port
-    streamAddr = strcpyrealloc(streamAddr, Location);
-    char *path = strstr(streamAddr, "/");
-    if (path) {
-       streamPath = strcpyrealloc(streamPath, path);
-       *path = 0;
-       }
-    else
-       streamPath = strcpyrealloc(streamPath, "/");
-    socketPort = Parameter;
-    debug("http://%s:%d%s\n", streamAddr, socketPort, streamPath);
-    // Re-connect the socket
-    Connect();
-    }
+     // Disconnect the current socket
+     Disconnect();
+     // Update stream address, path and port
+     streamAddr = strcpyrealloc(streamAddr, Location);
+     char *path = strstr(streamAddr, "/");
+     if (path) {
+        streamPath = strcpyrealloc(streamPath, path);
+        *path = 0;
+        }
+     else
+        streamPath = strcpyrealloc(streamPath, "/");
+     socketPort = Parameter;
+     debug("http://%s:%d%s\n", streamAddr, socketPort, streamPath);
+     // Re-connect the socket
+     Connect();
+     }
   return true;
 }
 
