@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: setup.c,v 1.50 2008/01/20 16:15:14 rahrenbe Exp $
+ * $Id: setup.c,v 1.56 2008/02/18 16:50:07 rahrenbe Exp $
  */
 
 #include <string.h>
@@ -37,12 +37,13 @@ private:
   struct tIptvChannel {
     int frequency, source, protocol, parameter, vpid, ppid, tpid, sid, nid, tid, rid;
     int apid[MAXAPIDS + 1], dpid[MAXDPIDS + 1], spid[MAXSPIDS + 1], caids[MAXCAIDS + 1];
+    int sidscan, pidscan;
     char name[256], location[256];
   } data;
   cChannel *channel;
   const char *protocols[eProtocolCount];
   void Setup(void);
-  cString GetIptvSettings(const char *Param, int *Parameter, int *Protocol);
+  cString GetIptvSettings(const char *Param, int *Parameter, int *SidScan, int *PidScan, int *Protocol);
   void GetChannelData(cChannel *Channel);
   void SetChannelData(cChannel *Channel);
 
@@ -69,12 +70,12 @@ cIptvMenuEditChannel::cIptvMenuEditChannel(cChannel *Channel, bool New)
   Setup();
 }
 
-cString cIptvMenuEditChannel::GetIptvSettings(const char *Param, int *Parameter, int *Protocol)
+cString cIptvMenuEditChannel::GetIptvSettings(const char *Param, int *Parameter, int *SidScan, int *PidScan, int *Protocol)
 {
   char *tag = NULL;
   char *proto = NULL;
   char *loc = NULL;
-  if (sscanf(Param, "%a[^|]|%a[^|]|%a[^|]|%d", &tag, &proto, &loc, Parameter) == 4) {
+  if (sscanf(Param, "%a[^|]|S%dP%d|%a[^|]|%a[^|]|%d", &tag, SidScan, PidScan, &proto, &loc, Parameter) == 6) {
      cString tagstr(tag, true);
      cString protostr(proto, true);
      cString locstr(loc, true);
@@ -95,13 +96,57 @@ cString cIptvMenuEditChannel::GetIptvSettings(const char *Param, int *Parameter,
         return locstr;
         }
      }
+  else if (sscanf(Param, "%a[^|]|P%dS%d|%a[^|]|%a[^|]|%d", &tag, PidScan, SidScan, &proto, &loc, Parameter) == 6) {
+     cString tagstr(tag, true);
+     cString protostr(proto, true);
+     cString locstr(loc, true);
+     // check if IPTV tag
+     if (strncasecmp(*tagstr, "IPTV", 4) == 0) {
+        // check if protocol is supported and update the pointer
+        if (strncasecmp(*protostr, "UDP", 3) == 0)
+           *Protocol = eProtocolUDP;
+        else if (strncasecmp(*protostr, "HTTP", 4) == 0)
+           *Protocol = eProtocolHTTP;
+        else if (strncasecmp(*protostr, "FILE", 4) == 0)
+           *Protocol = eProtocolFILE;
+        else if (strncasecmp(*protostr, "EXT", 3) == 0)
+           *Protocol = eProtocolEXT;
+        else
+           return NULL;
+        // return location
+        return locstr;
+        }
+     }
+  else if (sscanf(Param, "%a[^|]|%a[^|]|%a[^|]|%d", &tag, &proto, &loc, Parameter) == 4) {
+     cString tagstr(tag, true);
+     cString protostr(proto, true);
+     cString locstr(loc, true);
+     *SidScan = 0;
+     *PidScan = 0;
+     // check if IPTV tag
+     if (strncasecmp(*tagstr, "IPTV", 4) == 0) {
+        // check if protocol is supported and update the pointer
+        if (strncasecmp(*protostr, "UDP", 3) == 0)
+           *Protocol = eProtocolUDP;
+        else if (strncasecmp(*protostr, "HTTP", 4) == 0)
+           *Protocol = eProtocolHTTP;
+        else if (strncasecmp(*protostr, "FILE", 4) == 0)
+           *Protocol = eProtocolFILE;
+        else if (strncasecmp(*protostr, "EXT", 3) == 0)
+           *Protocol = eProtocolEXT;
+        else
+           return NULL;
+        // return location
+        return locstr;
+        }
+     }
   return NULL;
 }
 
 void cIptvMenuEditChannel::GetChannelData(cChannel *Channel)
 {
   if (Channel) {
-     int parameter, protocol;
+     int parameter, protocol, sidscan, pidscan;
      data.frequency = Channel->Frequency();
      data.source = Channel->Source();
      data.vpid = Channel->Vpid();
@@ -120,7 +165,9 @@ void cIptvMenuEditChannel::GetChannelData(cChannel *Channel)
      data.tid = Channel->Tid();
      data.rid = Channel->Rid();
      strn0cpy(data.name, Channel->Name(), sizeof(data.name));
-     strn0cpy(data.location, *GetIptvSettings(Channel->PluginParam(), &parameter, &protocol), sizeof(data.location));
+     strn0cpy(data.location, *GetIptvSettings(Channel->PluginParam(), &parameter, &sidscan, &pidscan, &protocol), sizeof(data.location));
+     data.sidscan = sidscan;
+     data.pidscan = pidscan;
      data.protocol = protocol;
      data.parameter = parameter;
      }
@@ -144,6 +191,8 @@ void cIptvMenuEditChannel::GetChannelData(cChannel *Channel)
      data.rid = 0;
      strn0cpy(data.name, "IPTV", sizeof(data.name));
      strn0cpy(data.location, "127.0.0.1", sizeof(data.location));
+     data.sidscan = 0;
+     data.pidscan = 0;
      data.protocol = eProtocolUDP;
      data.parameter = 1234;
      }
@@ -157,17 +206,17 @@ void cIptvMenuEditChannel::SetChannelData(cChannel *Channel)
      char dlangs[MAXDPIDS][MAXLANGCODE2] = { "" };
      switch (data.protocol) {
        case eProtocolEXT:
-            param = cString::sprintf("IPTV|EXT|%s|%d", data.location, data.parameter);
+            param = cString::sprintf("IPTV|S%dP%d|EXT|%s|%d", data.sidscan, data.pidscan, data.location, data.parameter);
             break;
        case eProtocolFILE:
-            param = cString::sprintf("IPTV|FILE|%s|%d", data.location, data.parameter);
+            param = cString::sprintf("IPTV|S%dP%d|FILE|%s|%d", data.sidscan, data.pidscan, data.location, data.parameter);
             break;
        case eProtocolHTTP:
-            param = cString::sprintf("IPTV|HTTP|%s|%d", data.location, data.parameter);
+            param = cString::sprintf("IPTV|S%dP%d|HTTP|%s|%d", data.sidscan, data.pidscan, data.location, data.parameter);
             break;
        default:
        case eProtocolUDP:
-            param = cString::sprintf("IPTV|UDP|%s|%d", data.location, data.parameter);
+            param = cString::sprintf("IPTV|S%dP%d|UDP|%s|%d", data.sidscan, data.pidscan, data.location, data.parameter);
             break;
        }
 #if defined(APIVERSNUM) && APIVERSNUM < 10510
@@ -218,6 +267,11 @@ void cIptvMenuEditChannel::Setup(void)
          Add(new cMenuEditIntItem(tr("Port"),   &data.parameter,  0, 0xFFFF));
          break;
     }
+  cOsdItem *sidScanItem = new cMenuEditBoolItem(tr("Scan Sid"), &data.sidscan);
+  if (!IptvConfig.GetSectionFiltering())
+     sidScanItem->SetSelectable(false);
+  Add(sidScanItem);
+  Add(new cMenuEditBoolItem(tr("Scan pids"),   &data.pidscan));
   // Normal settings
 #if defined(APIVERSNUM) && APIVERSNUM < 10511
   Add(new cMenuEditStrItem(trVDR("Name"),       data.name,     sizeof(data.name), trVDR(FileNameChars)));
@@ -257,44 +311,44 @@ eOSState cIptvMenuEditChannel::ProcessKey(eKeys Key)
         bool firstIncrement = true;
         // Search for identical channels as these will be ignored by vdr
         for (cChannel *iteratorChannel = Channels.First(); iteratorChannel;
-             iteratorChannel = Channels.Next(iteratorChannel)) {
-           // This is one of the channels cause the uniquity check to fail
-           if (!iteratorChannel->GroupSep() && iteratorChannel != channel
-               && iteratorChannel->GetChannelID() == newchannel.GetChannelID()) {
-              // See if it has unique Plugin param. If yes then increment
-              // the corresponding Rid until it is unique
-              if (strcmp(iteratorChannel->PluginParam(),
-                         newchannel.PluginParam())) {
-                 // If the channel RID is already at maximum, then fail the
-                 // channel modification
-                 if (iteratorChannel->Rid() >= 0x1FFF) {
-                    debug("Cannot increment RID over maximum value\n");
-                    uniquityFailed = true;
-                    break;
-                    }
-                 debug("Incrementing conflicting channel RID\n");
-                 iteratorChannel->SetId(iteratorChannel->Nid(),
-                                        iteratorChannel->Tid(),
-                                        iteratorChannel->Sid(),
-                                        firstIncrement ?
-                                        0 : iteratorChannel->Rid() + 1);
+            iteratorChannel = Channels.Next(iteratorChannel)) {
+            // This is one of the channels cause the uniquity check to fail
+            if (!iteratorChannel->GroupSep() && iteratorChannel != channel &&
+                iteratorChannel->GetChannelID() == newchannel.GetChannelID()) {
+               // See if it has unique Plugin param. If yes then increment
+               // the corresponding Rid until it is unique
+               if (strcmp(iteratorChannel->PluginParam(),
+                          newchannel.PluginParam())) {
+                  // If the channel RID is already at maximum, then fail the
+                  // channel modification
+                  if (iteratorChannel->Rid() >= 0x1FFF) {
+                     debug("Cannot increment RID over maximum value\n");
+                     uniquityFailed = true;
+                     break;
+                     }
+                  debug("Incrementing conflicting channel RID\n");
+                  iteratorChannel->SetId(iteratorChannel->Nid(),
+                                         iteratorChannel->Tid(),
+                                         iteratorChannel->Sid(),
+                                         firstIncrement ?
+                                         0 : iteratorChannel->Rid() + 1);
 
-                 // Try zero Rid:s at first increment. Prevents them from
-                 // creeping slowly towards their maximum value
-                 firstIncrement = false;
+                  // Try zero Rid:s at first increment. Prevents them from
+                  // creeping slowly towards their maximum value
+                  firstIncrement = false;
 
-                 // Re-set the search and start again
-                 iteratorChannel = Channels.First();
-                 continue;
-                 // Cannot work around by incrementing rid because channels
-                 // are actually copies of each other
-                 }
-              else {
-                 uniquityFailed = true;
-                 break;
-                 }
-              }
-           }
+                  // Re-set the search and start again
+                  iteratorChannel = Channels.First();
+                  continue;
+                  // Cannot work around by incrementing rid because channels
+                  // are actually copies of each other
+                  }
+               else {
+                  uniquityFailed = true;
+                  break;
+                  }
+               }
+            }
         if (!uniquityFailed) {
            if (channel) {
               SetChannelData(channel);
@@ -363,9 +417,7 @@ cIptvMenuChannelItem::cIptvMenuChannelItem(cChannel *Channel)
 
 void cIptvMenuChannelItem::Set(void)
 {
-  char *buffer = NULL;
-  asprintf(&buffer, "%d\t%s", channel->Number(), channel->Name());
-  SetText(buffer, false);
+  SetText(cString::sprintf("%d\t%s", channel->Number(), channel->Name()));
 }
 
 // --- cIptvMenuChannels -----------------------------------------------------
@@ -566,7 +618,7 @@ void cIptvMenuInfo::Display(void)
 {
   cOsdMenu::Display();
   DisplayMenu()->SetText(text, true);
-  if (text)
+  if (*text)
      cStatus::MsgOsdTextItem(text);
 }
 
@@ -622,7 +674,6 @@ cIptvPluginSetup::cIptvPluginSetup()
   tsBufferPrefill = IptvConfig.GetTsBufferPrefillRatio();
   extProtocolBasePort = IptvConfig.GetExtProtocolBasePort();
   sectionFiltering = IptvConfig.GetSectionFiltering();
-  sidScanning = IptvConfig.GetSidScanning();
   numDisabledFilters = IptvConfig.GetDisabledFiltersCount();
   if (numDisabledFilters > SECTION_FILTER_TABLE_SIZE)
      numDisabledFilters = SECTION_FILTER_TABLE_SIZE;
@@ -664,11 +715,6 @@ void cIptvPluginSetup::Setup(void)
 #endif
 
   if (sectionFiltering) {
-     Add(new cMenuEditBoolItem(tr("Scan Sid automatically"), &sidScanning));
-#if defined(APIVERSNUM) && APIVERSNUM >= 10513
-     help.Append(tr("Define whether the service id shall be scanned automatically.\n\nRequires the section filtering. Automatic Sid scanning helps VDR to detect changed pids of streams."));
-#endif
-
      Add(new cMenuEditIntItem( tr("Disable filters"), &numDisabledFilters, 0, SECTION_FILTER_TABLE_SIZE));
 #if defined(APIVERSNUM) && APIVERSNUM >= 10513
      help.Append(tr("Define number of section filters to be disabled.\n\nCertain section filters might cause some unwanted behaviour to VDR such as time being falsely synchronized. By black-listing the filters here useful section data can be left intact for VDR to process."));
@@ -756,14 +802,12 @@ void cIptvPluginSetup::Store(void)
   SetupStore("TsBufferPrefill", tsBufferPrefill);
   SetupStore("ExtProtocolBasePort", extProtocolBasePort);
   SetupStore("SectionFiltering", sectionFiltering);
-  SetupStore("SidScanning", sidScanning);
   StoreFilters("DisabledFilters", disabledFilterIndexes);
   // Update global config
   IptvConfig.SetTsBufferSize(tsBufferSize);
   IptvConfig.SetTsBufferPrefillRatio(tsBufferPrefill);
   IptvConfig.SetExtProtocolBasePort(extProtocolBasePort);
   IptvConfig.SetSectionFiltering(sectionFiltering);
-  IptvConfig.SetSidScanning(sidScanning);
   for (int i = 0; i < SECTION_FILTER_TABLE_SIZE; ++i)
       IptvConfig.SetDisabledFilters(i, disabledFilterIndexes[i]);
 }
