@@ -60,9 +60,14 @@ namespace features
 	struct epgsearch
 	{
 		static const char* Plugin() { return "epgsearch"; }
-		static const char* MinVersion() { return "0.9.23.beta10"; }
+		static const char* MinVersion() { return "0.9.24.beta20"; }
 	};
 
+	struct streamdev_server
+	{
+		static const char* Plugin() { return "streamdev-server"; }
+		static const char* MinVersion() { return "none"; }
+	};
 } // namespace features
 
 } // namespace vdrlive
