@@ -94,27 +94,32 @@ void cN2Timer::Stop(void)
 
 // -- cMapMemHW ----------------------------------------------------------------
 
+#define CRC_POLY 0x8408 // ccitt poly
+
 cMapMemHW::cMapMemHW(void)
 :cMapMem(HW_OFFSET,HW_REGS)
 {
   cycles=0;
-  CRCvalue=0; CRCpos=0; CRCstarttime=0;
+  CRCvalue=0xffff; CRCpos=0; CRCstarttime=0; CRCinput=0; CRCupdate=false;
   GenCRC16Table();
   PRINTF(L_SYS_EMU,"mapmemhw: new HW map off=%04x size=%04x",offset,size);
 }
 
 void cMapMemHW::GenCRC16Table(void)
 {
-  unsigned char hi[256];
-    for(int i=0; i<256; ++i) {
-      unsigned short c = i;
-      for(int j=0; j<8; ++j) c = (c>>1) ^ ((c&1) ? 0x8408 : 0); // ccitt poly
-      crc16table[0xff-i] = c & 0xff;
-      hi[i] = c>>8;
-      }
-    for(int i=0; i<32; ++i)
-      for(int j=0; j<8; ++j)
-        crc16table[i*8+j] |= hi[(0x87+(i*8)-j)&0xff]<<8;
+  for(int i=0; i<256; i++) {
+    unsigned short c=i;
+    for(int j=0; j<8; j++) c=(c>>1)^((c&1)?CRC_POLY:0);
+    crc16table[i]=c;
+    }
+}
+
+unsigned short cMapMemHW::CRC(unsigned short crc, unsigned char val, int bits)
+{
+  if(bits>=8) return (crc>>8)^crc16table[(crc^val)&0xff];
+  if(bits>0) crc^=(val&(0xff>>(8-bits)));
+  for(int i=0; i<bits; i++) crc=(crc>>1)^((crc&1)?CRC_POLY:0);
+  return crc;
 }
 
 void cMapMemHW::AddCycles(unsigned int num)
@@ -128,6 +133,8 @@ unsigned char cMapMemHW::Get(unsigned short ea)
   if(ea<offset || ea>=offset+size) return 0;
   ea-=offset;
   switch(ea) {
+    case HW_SECURITY:
+      return (mem[ea]&0x70)|0x0F;
     case HW_TIMER0_CONTROL:
     case HW_TIMER1_CONTROL:
     case HW_TIMER2_CONTROL:
@@ -149,11 +156,18 @@ unsigned char cMapMemHW::Get(unsigned short ea)
       }
     case HW_CRC_DATA:
       {
-      unsigned char r=CRCvalue>>((CRCpos&1)<<3);
-      CRCpos=!CRCpos;
+      unsigned short crc;
+      if(CRCupdate) {
+        crc=CRC(CRCvalue,CRCinput,cycles-CRCstarttime-1);
+        if(cycles-CRCstarttime>=CRCCALC_DELAY) {
+          CRCvalue=crc; CRCupdate=false;
+          }
+        }
+      else crc=CRCvalue;
+      unsigned char r=(crc^0xffff)>>((CRCpos&1)<<3);
+      CRCpos^=1;
       return r;
       }
-    case HW_SECURITY:
     default:
       return mem[ea];
     }
@@ -164,9 +178,6 @@ void cMapMemHW::Set(unsigned short ea, unsigned char val)
   if(ea<offset || ea>=offset+size) return;
   ea-=offset;
   switch(ea) {
-    case HW_SECURITY:
-      mem[ea]=(val&0x70)|0x0F;
-      break;
     case HW_TIMER0_CONTROL:
     case HW_TIMER1_CONTROL:
     case HW_TIMER2_CONTROL:
@@ -178,17 +189,23 @@ void cMapMemHW::Set(unsigned short ea, unsigned char val)
       timer[TIMER_NUM(ea)].Latch(val);
       break;
     case HW_CRC_CONTROL:
-      mem[ea]=val;
-      if(!(val&CRC_DISABLED)) {
-        CRCvalue=0; CRCpos=0;
+      if((mem[ea]&CRC_DISABLED) && !(val&CRC_DISABLED)) {
+        CRCvalue=0xffff; CRCinput=0; CRCpos=0; CRCupdate=false;
         CRCstarttime=cycles-CRCCALC_DELAY;
         }
+      mem[ea]=val;
       break;
     case HW_CRC_DATA:
-      if(!(mem[HW_CRC_CONTROL]&CRC_DISABLED) && cycles-CRCstarttime>=CRCCALC_DELAY) {
-        CRCvalue=(CRCvalue>>8)^crc16table[(CRCvalue^val)&0xFF];
-        CRCpos=0;
-        CRCstarttime=cycles;
+      if(cycles-CRCstarttime>=CRCCALC_DELAY) {
+        if(CRCupdate) {
+          CRCvalue=CRC(CRCvalue,CRCinput,8);
+          CRCupdate=false;
+          }
+        if(!(mem[HW_CRC_CONTROL]&CRC_DISABLED)) {
+          CRCinput=val;
+          CRCupdate=true;
+          CRCstarttime=cycles;
+          }
         }
       break;
     default:
@@ -231,22 +248,21 @@ bool cN2Emu::Init(int id, int romv)
   return initDone;
 }
 
-// -- cMapCore -----------------------------------------------------------------
+// -- cMapMath -----------------------------------------------------------------
 
-cMapCore::cMapCore(void)
+cMapMath::cMapMath(void)
 {
-  wordsize=4; last=1;
-  regs[0]=&J; regs[1]=&A; regs[2]=&B; regs[3]=&C; regs[4]=&D;
+  wordsize=DEF_WORDSIZE;
 }
 
-void cMapCore::ModAdd(BIGNUM *r, BIGNUM *a, BIGNUM *b, BIGNUM *d)
+void cMapMath::ModAdd(BIGNUM *r, BIGNUM *a, BIGNUM *b, BIGNUM *d)
 {
   BN_add(r,a,b);
   if(BN_cmp(r,d)>=0) BN_sub(r,r,d);
   BN_mask_bits(r,wordsize<<6);
 }
 
-void cMapCore::ModSub(BIGNUM *r, BIGNUM *d, BIGNUM *b)
+void cMapMath::ModSub(BIGNUM *r, BIGNUM *d, BIGNUM *b)
 {
   cBN p;
   BN_set_bit(p,wordsize<<6);
@@ -254,7 +270,7 @@ void cMapCore::ModSub(BIGNUM *r, BIGNUM *d, BIGNUM *b)
   BN_mask_bits(r,wordsize<<6);
 }
 
-void cMapCore::MakeJ0(BIGNUM *j, BIGNUM *d)
+void cMapMath::MakeJ0(BIGNUM *j, BIGNUM *d, int bits)
 {
 #if OPENSSL_VERSION_NUMBER < 0x0090700fL
 #error BN_mod_inverse is probably buggy in your openssl version
@@ -262,50 +278,85 @@ void cMapCore::MakeJ0(BIGNUM *j, BIGNUM *d)
   BN_zero(x);
   BN_sub(j,x,d);
   BN_set_bit(j,0);
-  BN_set_bit(x,64);
+  BN_set_bit(x,bits);
   BN_mod_inverse(j,j,x,ctx);
 }
 
-void cMapCore::MonMul(BIGNUM *o, BIGNUM *a, BIGNUM *b, BIGNUM *c, BIGNUM *d, BIGNUM *j, int words)
+void cMapMath::MonMul(BIGNUM *o, BIGNUM *a, BIGNUM *b)
 {
-  if(!words) words=wordsize;
-  BN_zero(s);
-  for(int i=0; i<words;) {
+  MonMul(o,a,b,C,D,J,0);
+}
+
+void cMapMath::MonMul(BIGNUM *o, BIGNUM *a, BIGNUM *b, int w)
+{
+  MonMul(o,a,b,C,D,J,w);
+}
+
+void cMapMath::MonMul(BIGNUM *o, BIGNUM *a, BIGNUM *b, BIGNUM *c, BIGNUM *d, BIGNUM *j, int w)
+{
+  if(!w) w=wordsize;
+  MonStart(w);
+  int i=0;
+  while(words>0) {
     BN_rshift(x,a,(i++)<<6);
-    BN_mask_bits(x,64);
-    BN_mul(x,x,b,ctx);
-    BN_add(s,s,x);
-
-    BN_copy(x,s);
-    BN_mask_bits(x,64);
-    BN_mul(x,x,j,ctx);
-    if(i==words) {
-      BN_lshift(y,x,64);
-      BN_add(y,y,x);
-      // Low
-      BN_rshift(c,y,2);
-      BN_add(c,c,s);
-      BN_rshift(c,c,52);
-      BN_mask_bits(c,12);
-      }
-
-    BN_mask_bits(x,64);
-    BN_mul(x,x,d,ctx);
-    BN_add(s,s,x);
-    if(i==words) {
-      // High
-      BN_lshift(y,s,12);
-      BN_add(c,c,y);
-      BN_mask_bits(c,wordsize<<6);
-      }
-
-    BN_rshift(s,s,64);
-    if(BN_cmp(s,d)==1) {
-      BN_copy(x,s);
-      BN_sub(s,x,d);
-      }
+    MonLoop(o,x,b,c,d,j);
     }
-  BN_copy(o,s);
+}
+
+void cMapMath::MonStart(int w)
+{
+  if(words<=0) {
+    words=w;
+    BN_zero(s);
+    }
+}
+
+// modifies a, but pointing a to x is allowed !!
+void cMapMath::MonLoop(BIGNUM *o, BIGNUM *a, BIGNUM *b, BIGNUM *c, BIGNUM *d, BIGNUM *j)
+{
+  words--;
+  BN_mask_bits(a,64);
+  BN_mul(a,a,b,ctx);
+  BN_add(s,s,a);
+
+  BN_copy(x,s);
+  BN_mask_bits(x,64);
+  BN_mul(x,x,j,ctx);
+  if(!words) {
+    BN_lshift(y,x,64);
+    BN_add(y,y,x);
+    // Low
+    BN_rshift(c,y,2);
+    BN_add(c,c,s);
+    BN_rshift(c,c,52);
+    BN_mask_bits(c,12);
+    }
+
+  BN_mask_bits(x,64);
+  BN_mul(x,x,d,ctx);
+  BN_add(s,s,x);
+  if(!words) {
+    // High
+    BN_lshift(y,s,12);
+    BN_add(c,c,y);
+    BN_mask_bits(c,wordsize<<6);
+    }
+
+  BN_rshift(s,s,64);
+  if(BN_cmp(s,d)==1) {
+    BN_copy(x,s);
+    BN_sub(s,x,d);
+    }
+
+  if(!words) BN_copy(o,s);
+}
+
+// -- cMapCore -----------------------------------------------------------------
+
+cMapCore::cMapCore(void)
+{
+  last=1;
+  regs[0]=&J; regs[1]=&A; regs[2]=&B; regs[3]=&C; regs[4]=&D;
 }
 
 void cMapCore::MonInit(int bits)
@@ -321,7 +372,6 @@ void cMapCore::MonInit(int bits)
 void cMapCore::MonExpNeg(void)
 {
   if(BN_is_zero(D)) { BN_set_word(A,1); return; }
-  cBN e;
   BN_copy(e,D);
   BN_mask_bits(e,8);		// check LSB
   unsigned int n=BN_get_word(e);
@@ -333,6 +383,14 @@ void cMapCore::MonExpNeg(void)
     MonMul(B,B,B);
     if(BN_is_bit_set(e,i)) MonMul(B,A,B);
     }
+  if(BN_is_bit_set(D,0)) {
+    int i;
+    for(i=BN_num_bits(D)-2; i>0; i--) if(BN_is_bit_set(D,i)) break;
+    if(i<=0) {
+      MonMul(B,B,B);
+      MonMul(B,A,B);
+      }
+    }
   BN_set_word(A,1);
   MonMul(B,A,B);
 }
@@ -443,48 +501,70 @@ void cMapCore::CurveInit(BIGNUM *a)
   BN_copy(A,B);
   BN_copy(I,D);
   BN_add_word(I,1);
+  BN_mask_bits(I,wordsize<<6);
   BN_rshift(I,I,1);
   MonMul(s140,I,B);
   MonMul(s160,B,a);
 }
 
+int cMapCore::GetOpSize(int l)
+{
+  return l!=0 ? l : wordsize;
+}
+
 bool cMapCore::DoMap(int f, unsigned char *data, int l)
 {
-  const int l1=l?l:wordsize;
+  const int l1=GetOpSize(l);
   const int dl=l1<<3;
   cycles=0;
   switch(f) {
     case SETSIZE:
-      wordsize=l; cycles=475-6; break;
+      cycles=(l>17 ? 459 : (l ? 475 : 454))-6;
+      if(l>=1 && l<=17) wordsize=l;
+      break;
 
     case IMPORT_J:
       cycles=890-6;
-      // fall through
+      last=0;
+      regs[0]->GetLE(data,8);
+      break;
     case IMPORT_A:
     case IMPORT_B:
     case IMPORT_C:
     case IMPORT_D:
-      if(!cycles) cycles=771+160*l1-6+(l==0?4:0);
+      if(l>17) { l=17; cycles+=5; }
+      else if(l<=0) { l=wordsize; cycles+=4; }
+      cycles+=771+160*l-6;
       last=f-IMPORT_J;
-      // fall through
+      regs[last]->GetLE(data,l<<3);
+      break;
     case IMPORT_LAST:
-      if(!cycles) cycles=656+160*l-6; // Even for 'J' cycles is dependent on 'l'
-      regs[last]->GetLE(data,last>0?dl:8);
+      if(l>16) { l=1; cycles+=5; }
+      else if(l<=0) l=1;
+      cycles=656+160*l-6;
+      regs[last]->GetLE(data,(last==0?1:l)<<3);
       break;
 
     case EXPORT_J:
       cycles=897-6;
-      // fall through
+      last=0;
+      regs[0]->PutLE(data,8);
+      break;
     case EXPORT_A:
     case EXPORT_B:
     case EXPORT_C:
     case EXPORT_D:
-      if(!cycles) cycles=778+160*l1-6+(l==0?4:0);
+      if(l>17) { l=17; cycles+=5; }
+      else if(l<=0) { l=wordsize; cycles+=4; }
+      cycles=778+160*l-6;
       last=f-EXPORT_J;
-      // fall through
+      regs[last]->PutLE(data,l<<3);
+      break;
     case EXPORT_LAST:
-      if(!cycles) cycles=668+160*l-6; // Even for 'J' cycles is dependent on 'l'
-      regs[last]->PutLE(data,last>0?dl:8);
+      if(l>16) { l=1; cycles+=5; }
+      else if(l<=0) l=1;
+      cycles=668+160*l-6;
+      regs[last]->PutLE(data,(last==0?1:l)<<3);
       break;
 
     case SWAP_A:
@@ -493,31 +573,31 @@ bool cMapCore::DoMap(int f, unsigned char *data, int l)
     case SWAP_D:
       cycles=776+248*l1-6;
       last=f-SWAP_A+1;
-      x.GetLE(data,dl);
+      e.GetLE(data,dl);
       regs[last]->PutLE(data,dl);
-      BN_copy(*regs[last],x);
+      BN_copy(*regs[last],e);
       break;
 
     case CLEAR_A:
     case CLEAR_B:
     case CLEAR_C:
     case CLEAR_D:
-      cycles=467+5*l1-6;
+      cycles=462+(8*l1+3)/5*5-6;
       last=f-CLEAR_A+1; BN_zero(*regs[last]);
       break;
 
     case COPY_A_B:
-      last=2; BN_copy(B,A); cycles=467+5*l1-6; break;
+      last=2; BN_copy(B,A); cycles=462+(8*l1+3)/5*5-6; break;
     case COPY_B_A:
-      last=1; BN_copy(A,B); cycles=467+5*l1-6; break;
+      last=1; BN_copy(A,B); cycles=462+(8*l1+3)/5*5-6; break;
     case COPY_A_C:
-      last=3; BN_copy(C,A); cycles=467+5*l1-6; break;
+      last=3; BN_copy(C,A); cycles=462+(8*l1+3)/5*5-6; break;
     case COPY_C_A:
-      last=1; BN_copy(A,C); cycles=467+5*l1-6; break;
+      last=1; BN_copy(A,C); cycles=462+(8*l1+3)/5*5-6; break;
     case COPY_C_D:
-      last=4; BN_copy(D,C); cycles=467+5*l1-6; break;
+      last=4; BN_copy(D,C); cycles=462+(8*l1+3)/5*5-6; break;
     case COPY_D_C:
-      last=3; BN_copy(C,D); cycles=467+5*l1-6; break;
+      last=3; BN_copy(C,D); cycles=462+(8*l1+3)/5*5-6; break;
 
     case 0x43: // init SHA1
       SHA1_Init(&sctx);
@@ -697,7 +777,7 @@ void cNagra2::CreatePQ(const unsigned char *key, BIGNUM *p, BIGNUM *q)
   idata[48] |= 0x80;
   idata[95] |= 1;
   BN_bin2bn(idata+48,48,q);
-  BN_add_word(q,(key[22] &0xf << 9 ) | (key[23]<<1));
+  BN_add_word(q,((key[22]&0xf)<<9) | (key[23]<<1));
 }
 
 bool cNagra2::Signature(const unsigned char *vkey, const unsigned char *sig, const unsigned char *msg, int len)
