@@ -42,6 +42,10 @@
 //#define TRUST_BOXTIME // should we trust the local time on this machine?
 //#define DUMP_TPSAU "/var/tmp"
 
+#ifdef DUMP_TPSAU
+#include <unistd.h>
+#endif
+
 // -- cRC6 ---------------------------------------------------------------------
 
 /*
@@ -172,6 +176,8 @@ time_t cTransponderTime::Now(void)
 
 // -- cSatTimeHook -------------------------------------------------------------
 
+#ifndef TESTER
+
 class cSatTimeHook : public cLogHook {
 private:
   cTransponderTime *ttime;
@@ -210,6 +216,8 @@ void cSatTimeHook::Process(int pid, unsigned char *data)
     }
 }
 
+#endif //TESTER
+
 // -- cSatTime -----------------------------------------------------------------
 
 class cSatTime {
@@ -278,6 +286,7 @@ private:
   void Dump(void);
 public:
   cOpenTVModule(const unsigned char *data);
+  cOpenTVModule(int Id, const unsigned char *data, int len);
   ~cOpenTVModule();
   int AddPart(const unsigned char *data, int len);
   const info_header_t *InfoHdr(void) const { return &info; }
@@ -296,6 +305,14 @@ cOpenTVModule::cOpenTVModule(const unsigned char *data)
   mem=MALLOC(unsigned char,modlen);
 }
 
+cOpenTVModule::cOpenTVModule(int Id, const unsigned char *data, int len)
+{
+  id=Id; modlen=received=len;
+  mem=MALLOC(unsigned char,modlen);
+  memcpy(mem,data,len);
+  ParseSections();
+}
+
 cOpenTVModule::~cOpenTVModule()
 {
   free(mem);
@@ -552,6 +569,8 @@ cString cTpsKey::ToString(bool hide)
 
 // -- cTpsAuHook ---------------------------------------------------------------
 
+#ifndef TESTER
+
 #define BUFF_SIZE 20000
 
 class cTpsAuHook : public cLogHook {
@@ -564,6 +583,8 @@ public:
   virtual void Process(int pid, unsigned char *data);
   };
 
+#endif //TESTER
+
 // -- cTpsKeys -----------------------------------------------------------------
 
 cTpsKeys tpskeys;
@@ -730,10 +751,18 @@ bool cTpsKeys::ProcessAu(const cOpenTVModule *mod)
   const unsigned char *d=datahdr->data;
   unsigned int kd=0, cb1=0, cb2=0, cb3=0;
   for(unsigned int i=0; i<codehdr->size; i++) {
-    if(c[i] == 0x81) { // PushEA DS:$xxxx
+    if(c[i]==0x81) { // PushEA DS:$xxxx
       unsigned int addr=(c[i+1]<<8)|c[i+2];
       if(addr<(datahdr->dlen-3)) {
-        if(d[addr+1]==0x00 && d[addr+3]==0x00 && d[addr+4]==3) kd=addr;
+        if(d[addr]==0x79 && d[addr+1]==0x00 && d[addr+2]==0x79 && d[addr+3]==0x00)
+          kd=addr;
+        else if(d[addr]==0x73 && d[addr+1]==0x25 && d[addr+2]==0xFA)
+          cb1=addr;
+        else if(d[addr]==0x64 && (d[addr+1]&0xB0)==0xB0 && d[addr+2]==0x24)
+          cb2=addr;
+        else if((d[addr]&0x60)==0x60 && (d[addr+1]&0xB0)==0xB0 && (d[addr+2]&0x20)==0x20)
+          cb3=addr;
+/*
         else if(d[addr]==0x73 && d[addr+1]==0x25) {
           static const unsigned char scan1[] = { 0x28, 0x20, 0x20, 0xC0 };
           for(int j=2; j < 0xC; j++)
@@ -741,32 +770,19 @@ bool cTpsKeys::ProcessAu(const cOpenTVModule *mod)
           }
         else if(cb1 && !cb2) cb2=addr;
         else if(cb1 && cb2 && !cb3) cb3=addr;
-/*
-        else if((d[addr]&0xF0)==0x60 && (d[addr+1]&0xF0)==0xB0) {
-          int vajw = (int)(((~(d[addr]&0x0F))<<4)|(d[addr+1]&0x0F));
-          unsigned char hits=0;
-          for(int j=2; j < 0x30; j++) {
-            int vld = ((d[addr+j]&0x0F)<<4)|(d[addr+j+1]&0x0F);
-            if((d[addr+j]&0xF0)==0x20 && (d[addr+j+1]&0xF0)==0x70) {
-              int val=vajw+vld;
-              if(val==3 || val==4 || val==5) hits++;
-              }
-            }
-          if(hits==3) cb3=addr;
-          else if(hits==2) {
-            if(cb2==0) cb2=addr;
-            else if(cb3==0) cb3=addr;
-            }
-          }
 */
         }
       }
     }
   if(!kd || !cb1 || !cb2 || !cb3) {
-    PRINTF(L_SYS_TPSAU,"couldn't locate all pointers in data section");
+    PRINTF(L_SYS_TPSAU,"couldn't locate all pointers in data section (%d,%d,%d,%d)",kd,cb1,cb2,cb3);
     return false;
     }
-  RegisterAlgo3(d,cb1,cb2,cb3,kd);
+
+  unsigned int end=(kd>cb1 && kd>cb2 && kd>cb3) ? kd : datahdr->dlen;
+  unsigned int off=min(cb1,min(cb2,cb3))-2;
+  PRINTF(L_SYS_TPSAU,"pointers in data section kd=%d cb1=%d cb2=%d cb3=%d - dlen=%d off=%d end=%d - cb1=%d cb2=%d cb3=%d",kd,cb1,cb2,cb3,datahdr->dlen,off,end,cb1-off,cb2-off,cb3-off);
+  RegisterAlgo3(d+off,cb1-off,cb2-off,cb3-off,end-off);
 
   const unsigned char *data=&d[kd];
   int seclen, numkeys;
@@ -901,7 +917,7 @@ void cTpsKeys::PreLoad(void)
 
 void cTpsKeys::PostLoad(void)
 {
-  if(loadlist) Join(loadlist);
+  if(loadlist) { Join(loadlist); loadlist=0; }
 }
 
 bool cTpsKeys::ParseLinePlain(const char *line)
@@ -976,6 +992,8 @@ void cTpsKeys::PostSave(FILE *f)
 
 // -- cTpsAuHook ---------------------------------------------------------------
 
+#ifndef TESTER
+
 #define AUSID 0x12C0
 
 cTpsAuHook::cTpsAuHook(void)
@@ -1052,6 +1070,8 @@ void cTpsAuHook::Process(int pid, unsigned char *data)
     }
 }
 
+#endif //TESTER
+
 // -- cTPSDecrypt --------------------------------------------------------------
 
 unsigned char *cTPSDecrypt::mem=0;
@@ -1079,13 +1099,13 @@ void cTPSDecrypt::TpsDecrypt(unsigned char *data, short mode, const unsigned cha
     }
 }
 
-bool cTPSDecrypt::RegisterAlgo3(const unsigned char *data, int cb1, int cb2, int cb3, int kd)
+bool cTPSDecrypt::RegisterAlgo3(const unsigned char *data, int cb1, int cb2, int cb3, int len)
 {
   cMutexLock lock(&st20Mutex);
   free(mem);
-  if(!(mem=MALLOC(unsigned char,kd))) return false;
-  memcpy(mem,data,kd);
-  memLen=kd; cb1off=cb1; cb2off=cb2; cb3off=cb3;
+  if(!(mem=MALLOC(unsigned char,len))) return false;
+  memcpy(mem,data,len);
+  memLen=len; cb1off=cb1; cb2off=cb2; cb3off=cb3;
   st20Inited=false;
   PRINTF(L_SYS_TPSAU,"registered callbacks for algo 3");
   return true;
