@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c,v 1.49 2008/01/10 23:36:06 phelin Exp $
+ * $Id: config.c,v 1.56 2008/03/13 22:14:06 phintuka Exp $
  *
  */
 
@@ -35,7 +35,7 @@ const int config_t::i_pesBufferSize[ PES_BUFFERS_count+1 ] =  {
   0, 50, 250, 500, 1000, 2000, 500
 };
 
-const char *config_t::s_bufferSize[ PES_BUFFERS_count+1 ] = { 
+const char * const config_t::s_bufferSize[ PES_BUFFERS_count+1 ] = { 
   trNOOP("custom"),
   trNOOP("tiny"),
   trNOOP("small"),
@@ -45,7 +45,7 @@ const char *config_t::s_bufferSize[ PES_BUFFERS_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_aspects[ ASPECT_count+1 ] = {
+const char * const config_t::s_aspects[ ASPECT_count+1 ] = {
   trNOOP("automatic"),
   trNOOP("default"),
   "4:3",
@@ -56,7 +56,17 @@ const char *config_t::s_aspects[ ASPECT_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_deinterlaceMethods[ DEINTERLACE_count+1 ] = {
+const char * const config_t::s_vo_aspects[ VO_ASPECT_count+1 ] = {
+  trNOOP("automatic"),
+  trNOOP("square"),
+  "4:3",
+  trNOOP("anamorphic"),
+  trNOOP("DVB"),
+  NULL
+};
+
+
+const char * const config_t::s_deinterlaceMethods[ DEINTERLACE_count+1 ] = {
   "none",
   "bob",
   "weave",
@@ -68,7 +78,7 @@ const char *config_t::s_deinterlaceMethods[ DEINTERLACE_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_deinterlaceMethodNames[ DEINTERLACE_count+1 ] = {
+const char * const config_t::s_deinterlaceMethodNames[ DEINTERLACE_count+1 ] = {
   trNOOP("off"),
   "Bob",
   "Weave",
@@ -80,18 +90,18 @@ const char *config_t::s_deinterlaceMethodNames[ DEINTERLACE_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_fieldOrder[ FIELD_ORDER_count+1 ] = { 
+const char * const config_t::s_fieldOrder[ FIELD_ORDER_count+1 ] = { 
   trNOOP("normal"),
   trNOOP("inverted"),
   NULL
 };
 
-const char *config_t::s_audioDrivers[ AUDIO_DRIVER_count+1 ] = { 
+const char * const config_t::s_audioDrivers[ AUDIO_DRIVER_count+1 ] = { 
   "auto", "alsa", "oss", "none", "esd", "jack",
   NULL
 };
 
-const char *config_t::s_audioDriverNames[ AUDIO_DRIVER_count+1 ] = {
+const char * const config_t::s_audioDriverNames[ AUDIO_DRIVER_count+1 ] = {
   trNOOP("automatic"),
   "Alsa",
   "OSS",
@@ -101,12 +111,12 @@ const char *config_t::s_audioDriverNames[ AUDIO_DRIVER_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_videoDriversX11[ X11_DRIVER_count+1 ] =  {
+const char * const config_t::s_videoDriversX11[ X11_DRIVER_count+1 ] =  {
   "auto", "xshm", "xv", "xvmc", "xxmc", "vidix", "XDirectFB", "opengl", "sdl", "none",
   NULL
 };
 
-const char *config_t::s_videoDriverNamesX11[ X11_DRIVER_count+1 ] =  {
+const char * const config_t::s_videoDriverNamesX11[ X11_DRIVER_count+1 ] =  {
   trNOOP("automatic"),
   "XShm",
   "Xv",
@@ -120,12 +130,12 @@ const char *config_t::s_videoDriverNamesX11[ X11_DRIVER_count+1 ] =  {
   NULL
 };
 
-const char *config_t::s_videoDriversFB[ FB_DRIVER_count+1 ] = {
+const char * const config_t::s_videoDriversFB[ FB_DRIVER_count+1 ] = {
   "auto", "fb", "DirectFB", "sdl", "vidixfb", "aadxr3", "none",
   NULL 
 };
 
-const char *config_t::s_videoDriverNamesFB [ FB_DRIVER_count+1 ] = {
+const char * const config_t::s_videoDriverNamesFB [ FB_DRIVER_count+1 ] = {
   trNOOP("automatic"),
   "Framebuffer",
   "DirectFB",
@@ -136,19 +146,19 @@ const char *config_t::s_videoDriverNamesFB [ FB_DRIVER_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_frontends[ FRONTEND_count+1 ] = {
+const char * const config_t::s_frontends[ FRONTEND_count+1 ] = {
   "sxfe", "fbfe", "none",
   NULL
 };
 
-const char *config_t::s_frontendNames[ FRONTEND_count+1 ] = {
+const char * const config_t::s_frontendNames[ FRONTEND_count+1 ] = {
   "X11 (sxfe)",
   "Framebuffer (fbfe)",
   trNOOP("Off"),
   NULL 
 };
 
-const char *config_t::s_frontend_files[ FRONTEND_count+1 ] = {
+const char * const config_t::s_frontend_files[ FRONTEND_count+1 ] = {
   "lib" PLUGIN_NAME_I18N "-sxfe.so." XINELIBOUTPUT_VERSION,
   "lib" PLUGIN_NAME_I18N "-fbfe.so." XINELIBOUTPUT_VERSION,
   // example: libxineliboutput-sxfe.so.0.4.0
@@ -156,18 +166,18 @@ const char *config_t::s_frontend_files[ FRONTEND_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_audioEqNames[ AUDIO_EQ_count+1 ] = {
+const char * const config_t::s_audioEqNames[ AUDIO_EQ_count+1 ] = {
   "30 Hz", "60 Hz", "125 Hz", "250 Hz", "500 Hz",
   "1 kHz", "2 kHz", "4 kHz", "8 kHz", "16 kHz",
   NULL
 };
 
-const char *config_t::s_audioVisualizations[ AUDIO_VIS_count+1 ] = {
+const char * const config_t::s_audioVisualizations[ AUDIO_VIS_count+1 ] = {
   "none", "goom", "oscope", "fftscope", "fftgraph",
   NULL
 };
 
-const char *config_t::s_audioVisualizationNames[ AUDIO_VIS_count+1 ] = {
+const char * const config_t::s_audioVisualizationNames[ AUDIO_VIS_count+1 ] = {
   trNOOP("Off"),
   trNOOP("Goom"),
   trNOOP("Oscilloscope"),
@@ -177,7 +187,7 @@ const char *config_t::s_audioVisualizationNames[ AUDIO_VIS_count+1 ] = {
 };
 
 /* xine, audio_alsa_out.c */
-const char *config_t::s_speakerArrangements[ SPEAKERS_count+1 ] = {
+const char * const config_t::s_speakerArrangements[ SPEAKERS_count+1 ] = {
   trNOOP("Mono 1.0"), trNOOP("Stereo 2.0"), trNOOP("Headphones 2.0"), trNOOP("Stereo 2.1"),
   trNOOP("Surround 3.0"), trNOOP("Surround 4.0"), trNOOP("Surround 4.1"),
   trNOOP("Surround 5.0"), trNOOP("Surround 5.1"), trNOOP("Surround 6.0"),
@@ -185,7 +195,7 @@ const char *config_t::s_speakerArrangements[ SPEAKERS_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_subtitleSizes[ SUBTITLESIZE_count+1 ] = { 
+const char * const config_t::s_subtitleSizes[ SUBTITLESIZE_count+1 ] = { 
   trNOOP("default"),
   trNOOP("tiny"),
   trNOOP("small"),
@@ -196,17 +206,17 @@ const char *config_t::s_subtitleSizes[ SUBTITLESIZE_count+1 ] = {
   NULL
 };
 
-const char *config_t::s_subExts[] = {
+const char * const config_t::s_subExts[] = {
   ".sub", ".srt", ".txt", ".ssa",
   ".SUB", ".SRT", ".TXT", ".SSA", 
   NULL
 };
 
-const char *config_t::s_osdMixers[] = {
+const char * const config_t::s_osdMixers[] = {
   trNOOP("no"),
   trNOOP("grayscale"),              // item [1]
   trNOOP("transparent"),            // item [2]
-  trNOOP("transparent greyscale"),  // item [3] ([1 | 2])
+  trNOOP("transparent grayscale"),  // item [3] ([1 | 2])
   trNOOP("yes"),
   NULL
 };
@@ -327,63 +337,60 @@ bool config_t::IsImageFile(const char *fname)
   return false;
 }
 
-const char *config_t::AutocropOptions(void)
+cString config_t::AutocropOptions(void)
 {
-  if(autocrop) {
-    static char buffer[256];
-    snprintf(buffer, sizeof(buffer),
-	     "enable_autodetect=%d,soft_start=%d,stabilize=%d,enable_subs_detect=%d",
-	     autocrop_autodetect, autocrop_soft, autocrop_fixedsize, autocrop_subs);
-    buffer[sizeof(buffer)-1] = 0;
-    return buffer;
-  }
-  return NULL;
+  if (!autocrop)
+    return NULL;
+
+  return cString::sprintf("enable_autodetect=%d,soft_start=%d,stabilize=%d,enable_subs_detect=%d",
+			  autocrop_autodetect, autocrop_soft, autocrop_fixedsize, autocrop_subs);
 }
 
-const char *config_t::FfmpegPpOptions(void)
+cString config_t::SwScaleOptions(void)
 {
-  if(ffmpeg_pp) {
-    static char buffer[128];
-    if(*ffmpeg_pp_mode)
-      snprintf(buffer, sizeof(buffer), "quality=%d,mode=%s", ffmpeg_pp_quality, ffmpeg_pp_mode);
-    else
-      snprintf(buffer, sizeof(buffer), "quality=%d", ffmpeg_pp_quality);
-    buffer[sizeof(buffer)-1] = 0;
-    return buffer;
-  }
-  return NULL;
+  if (!swscale)
+    return NULL;
+
+  return cString::sprintf("output_aspect=%s,output_width=%d,output_height=%d,no_downscaling=%d",
+			  swscale_change_aspect ? "auto" : "0.0",
+			  swscale_resize ? swscale_width  : 0,
+			  swscale_resize ? swscale_height : 0,
+			  swscale_downscale ? 0 : 1 );
 }
 
-const char *config_t::UnsharpOptions(void)
+cString config_t::FfmpegPpOptions(void)
 {
-  if(unsharp) {
-    static char buffer[128];
-    snprintf(buffer, sizeof(buffer),
-             "luma_matrix_width=%d,luma_matrix_height=%d,luma_amount=%1.1f,"
-             "chroma_matrix_width=%d,chroma_matrix_height=%d,chroma_amount=%1.1f",
-             unsharp_luma_matrix_width, unsharp_luma_matrix_height,
-             ((float)unsharp_luma_amount)/10.0,
-             unsharp_chroma_matrix_width, unsharp_chroma_matrix_height,
-             ((float)unsharp_chroma_amount)/10.0);
-    buffer[sizeof(buffer)-1] = 0;
-    return buffer;
-  }
-  return NULL;
+  if (!ffmpeg_pp)
+    return NULL;
+
+  if(*ffmpeg_pp_mode)
+    return cString::sprintf("quality=%d,mode=%s", ffmpeg_pp_quality, ffmpeg_pp_mode);
+
+  return cString::sprintf("quality=%d", ffmpeg_pp_quality);
 }
 
-const char *config_t::Denoise3dOptions(void)
+cString config_t::UnsharpOptions(void)
 {
-  if(denoise3d) {
-    static char buffer[128];
-    snprintf(buffer, sizeof(buffer),
-             "luma=%1.1f,chroma=%1.1f,time=%1.1f",
-             ((float)denoise3d_luma)/10.0,
-             ((float)denoise3d_chroma)/10.0,
-             ((float)denoise3d_time)/10.0);
-    buffer[sizeof(buffer)-1] = 0;
-    return buffer;
-  }
-  return NULL;
+  if (!unsharp)
+    return NULL;
+
+  return cString::sprintf("luma_matrix_width=%d,luma_matrix_height=%d,luma_amount=%1.1f,"
+			  "chroma_matrix_width=%d,chroma_matrix_height=%d,chroma_amount=%1.1f",
+			  unsharp_luma_matrix_width, unsharp_luma_matrix_height,
+			  ((float)unsharp_luma_amount)/10.0,
+			  unsharp_chroma_matrix_width, unsharp_chroma_matrix_height,
+			  ((float)unsharp_chroma_amount)/10.0);
+}
+
+cString config_t::Denoise3dOptions(void)
+{
+  if (!denoise3d)
+    return NULL;
+
+  return cString::sprintf("luma=%1.1f,chroma=%1.1f,time=%1.1f",
+			  ((float)denoise3d_luma)/10.0,
+			  ((float)denoise3d_chroma)/10.0,
+			  ((float)denoise3d_time)/10.0);
 }
 
 config_t::config_t() {
@@ -443,12 +450,14 @@ config_t::config_t() {
   unscaled_osd_opaque  = 0;
   unscaled_osd_lowresvideo = 1;
 
+#if VDRVERSNUM < 10515
   spu_autoshow = 0;
   memset(spu_lang, 0, sizeof(spu_lang));
-  strn0cpy(spu_lang[0], "en", sizeof(spu_lang[0]));
-  strn0cpy(spu_lang[2], "de", sizeof(spu_lang[2]));
-  strn0cpy(spu_lang[1], "fi", sizeof(spu_lang[1]));
-  //strn0cpy(spu_lang[3], "" , sizeof(spu_lang[3]));
+  strn0cpy(spu_lang[0], "eng", sizeof(spu_lang[0]));
+  strn0cpy(spu_lang[1], "deu", sizeof(spu_lang[1]));
+  strn0cpy(spu_lang[2], "fin", sizeof(spu_lang[2]));
+  strn0cpy(spu_lang[3], "" ,   sizeof(spu_lang[3]));
+#endif
   extsub_size = -1;
 
   alpha_correction     = 0;
@@ -466,6 +475,13 @@ config_t::config_t() {
   autocrop_fixedsize = 1;
   autocrop_subs  = 1;
 
+  swscale               = 0;    // enable/disable
+  swscale_change_aspect = 0;    // change video aspect ratio
+  swscale_resize        = 0;    // change video size
+  swscale_width         = 720;  //   output video width
+  swscale_height        = 576;  //   output video height
+  swscale_downscale     = 0;    //   allow downscaling
+
   remote_mode    = 0;
   listen_port    = LISTEN_PORT;
   remote_keyboard = 1;
@@ -506,6 +522,7 @@ config_t::config_t() {
   saturation   = -1; 
   contrast     = -1; 
   brightness   = -1; 
+  vo_aspect_ratio = 0;
 
   strn0cpy(browse_files_dir,  VideoDirectory, sizeof(browse_files_dir));
   strn0cpy(browse_music_dir,  VideoDirectory, sizeof(browse_music_dir));
@@ -515,8 +532,6 @@ config_t::config_t() {
 
   main_menu_mode = ShowMenu;
   force_primary_device = 0;
-
-  m_ProcessedArgs = NULL;
 };
 
 static uint8_t g_hidden_options[sizeof(config_t)] = {0};
@@ -524,22 +539,21 @@ static uint8_t g_readonly_options[sizeof(config_t)] = {0};
 uint8_t *config_t::hidden_options   = &g_hidden_options[0];
 uint8_t *config_t::readonly_options = &g_readonly_options[0];
 
+cString config_t::m_ProcessedArgs;
 bool config_t::ProcessArg(const char *Name, const char *Value)
 {
-  char *s = m_ProcessedArgs;
-  m_ProcessedArgs = NULL;
   if(SetupParse(Name, Value)) {
-    asprintf(&m_ProcessedArgs, "%s%s ", s?s:" ", Name);
-    free(s);
+    m_ProcessedArgs = cString::sprintf("%s%s ", *m_ProcessedArgs ? *m_ProcessedArgs : " ", Name);
     return true;
   }
-  m_ProcessedArgs = s;
   return false;
 }
 
 bool config_t::ProcessArgs(int argc, char *argv[])
 {
-  static struct option long_options[] = {
+  static const char short_options[] = "fw:h:l:r:A:V:d:P:pc";
+
+  static const struct option long_options[] = {
       { "fullscreen",   no_argument,       NULL, 'f' },
       { "width",        required_argument, NULL, 'w' },
       { "height",       required_argument, NULL, 'h' },
@@ -561,7 +575,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
     };
 
   int c;
-  while ((c = getopt_long(argc, argv, "fw:h:l:r:A:V:d:P:pc", long_options, NULL)) != -1) {
+  while ((c = getopt_long(argc, argv, short_options, long_options, NULL)) != -1) {
     switch (c) {
     case 'd': ProcessArg("Video.Port", optarg);
               break;
@@ -634,7 +648,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
 bool config_t::SetupParse(const char *Name, const char *Value)
 {
   char *pt;
-  if(m_ProcessedArgs && NULL != (pt=strstr(m_ProcessedArgs+1, Name)) &&
+  if(*m_ProcessedArgs && NULL != (pt=strstr(m_ProcessedArgs+1, Name)) &&
      *(pt-1) == ' ' && *(pt+strlen(Name)) == ' ') {
     LOGDBG("Skipping configuration entry %s=%s (overridden in command line)", Name, Value);
     return true;
@@ -675,11 +689,13 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "OSD.AlphaCorrection"))    alpha_correction = atoi(Value);
   else if (!strcasecmp(Name, "OSD.AlphaCorrectionAbs")) alpha_correction_abs = atoi(Value);
 
+#if VDRVERSNUM < 10515
   else if (!strcasecmp(Name, "OSD.SpuAutoSelect")) spu_autoshow = atoi(Value);
   else if (!strcasecmp(Name, "OSD.SpuLang0")) STRN0CPY(spu_lang[0], Value);
   else if (!strcasecmp(Name, "OSD.SpuLang1")) STRN0CPY(spu_lang[1], Value);
   else if (!strcasecmp(Name, "OSD.SpuLang2")) STRN0CPY(spu_lang[2], Value);
   else if (!strcasecmp(Name, "OSD.SpuLang3")) STRN0CPY(spu_lang[3], Value);
+#endif
   else if (!strcasecmp(Name, "OSD.ExtSubSize"))    extsub_size = atoi(Value);
 
   else if (!strcasecmp(Name, "RemoteMode"))          remote_mode = atoi(Value);
@@ -719,6 +735,14 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Video.AutoCrop.SoftStart"))    autocrop_soft = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.FixedSize"))    autocrop_fixedsize = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.DetectSubs"))   autocrop_subs = atoi(Value);
+
+  else if (!strcasecmp(Name, "Video.SwScale"))           swscale = atoi(Value);
+  else if (!strcasecmp(Name, "Video.SwScale.Aspect"))    swscale_change_aspect = atoi(Value);
+  else if (!strcasecmp(Name, "Video.SwScale.Resize"))    swscale_resize = atoi(Value);
+  else if (!strcasecmp(Name, "Video.SwScale.Downscale")) swscale_downscale = atoi(Value);
+  else if (!strcasecmp(Name, "Video.SwScale.Width"))     swscale_width = atoi(Value);
+  else if (!strcasecmp(Name, "Video.SwScale.Height"))    swscale_height = atoi(Value);
+
   else if (!strcasecmp(Name, "Video.HUE"))         hue = atoi(Value);
   else if (!strcasecmp(Name, "Video.Saturation"))  saturation = atoi(Value);
   else if (!strcasecmp(Name, "Video.Contrast"))    contrast = atoi(Value);
@@ -726,6 +750,7 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Video.Overscan"))    overscan = atoi(Value);
   else if (!strcasecmp(Name, "Video.IBPTrickSpeed"))  ibp_trickspeed = atoi(Value);
   else if (!strcasecmp(Name, "Video.MaxTrickSpeed"))  max_trickspeed = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AspectRatio"))    vo_aspect_ratio = atoi(Value);
 
   else if (!strcasecmp(Name, "Post.pp.Enable"))    ffmpeg_pp = atoi(Value);
   else if (!strcasecmp(Name, "Post.pp.Quality"))   ffmpeg_pp_quality = atoi(Value);
