@@ -4,11 +4,12 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c,v 1.49 2008/01/24 09:31:05 phintuka Exp $
+ * $Id: device.c,v 1.54 2008/03/16 21:40:57 phintuka Exp $
  *
  */
 
 #define __STDC_FORMAT_MACROS
+#define __STDC_CONSTANT_MACROS
 #include <inttypes.h>
 
 #include <vdr/config.h>
@@ -22,12 +23,10 @@
 //#define XINELIBOUTPUT_DEBUG
 //#define XINELIBOUTPUT_DEBUG_STDERR
 //#define TRACK_EXEC_TIME
-//#define SKIP_DVDSPU
 //#define FORWARD_DVD_SPUS
 //#define DEBUG_SWITCHING_TIME
 //#define LOG_TRICKSPEED
 #if VDRVERSNUM >= 10510
-# define SKIP_DVDSPU
 # define DEVICE_SUPPORTS_IBP_TRICKSPEED
 #endif
 
@@ -197,14 +196,15 @@ cXinelibDevice::cXinelibDevice()
   m_ac3Present  = false;
   m_spuPresent  = false;
 
-  m_CurrentDvdSpuTrack = -1;
+#if VDRVERSNUM < 10515
+  m_CurrentDvdSpuTrack = ttXSubtitleNone;
   m_ForcedDvdSpuTrack = false;
   ClrAvailableDvdSpuTracks();
+#endif
 
   memset(m_MetaInfo, 0, sizeof(m_MetaInfo));
     
   m_PlayMode = pmNone;
-  m_LastTrack = ttAudioFirst;
   m_AudioChannel = 0;
 
   m_liveMode    = true;
@@ -220,6 +220,7 @@ cXinelibDevice::cXinelibDevice()
   m_AudioCount  = 0;
   m_FreeBufs = 0;
   m_Cleared = true;
+  m_h264 = false;
 }
 
 cXinelibDevice::~cXinelibDevice() 
@@ -454,14 +455,14 @@ void cXinelibDevice::ConfigurePostprocessing(const char *name, bool on,
 }
 
 void cXinelibDevice::ConfigureVideo(int hue, int saturation, int brightness, int contrast,
-				    int overscan)
+				    int overscan, int vo_aspect_ratio)
 {
   TRACEF("cXinelibDevice::ConfigureVideo");
 
   if(m_local)
-    m_local->ConfigureVideo(hue, saturation, brightness, contrast, overscan);
+    m_local->ConfigureVideo(hue, saturation, brightness, contrast, overscan, vo_aspect_ratio);
   if(m_server)
-    m_server->ConfigureVideo(hue, saturation, brightness, contrast, overscan);
+    m_server->ConfigureVideo(hue, saturation, brightness, contrast, overscan, vo_aspect_ratio);
 }
 
 void cXinelibDevice::ConfigureDecoder(int pes_buffers)
@@ -586,8 +587,10 @@ void cXinelibDevice::StopOutput(void)
   Clear();
   ForEach(m_clients, &cXinelibThread::QueueBlankDisplay);
   ForEach(m_clients, &cXinelibThread::SetNoVideo, false);
+#if VDRVERSNUM < 10515
   ClrAvailableDvdSpuTracks();
   m_ForcedDvdSpuTrack = false;
+#endif
 }
 
 void cXinelibDevice::SetTvMode(cChannel *Channel)
@@ -657,7 +660,9 @@ bool cXinelibDevice::SetPlayMode(ePlayMode PlayMode)
   m_ac3Present = false;
   m_spuPresent = false;
 
+#if VDRVERSNUM < 10515
   ClrAvailableDvdSpuTracks();
+#endif
   m_PlayMode = PlayMode;
 
   TrickSpeed(-1);
@@ -818,6 +823,7 @@ void cXinelibDevice::Clear(void)
     //LOGMSG("************ FIRST  Clear ***************");
     m_Cleared = true;
     m_StreamStart = true;
+    m_h264 = false;
     m_FreeBufs = 0;
     TrickSpeed(-1);
     ForEach(m_clients, &cXinelibThread::Clear);
@@ -938,18 +944,15 @@ bool cXinelibDevice::PlayFile(const char *FileName, int Position,
 int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length) 
 {
   if(abs(m_TrickSpeed) > 1 && (m_TrickSpeedMode & trs_I_frames)) {
-    uchar PictureType = NO_PICTURE;
-    if(ScanVideoPacket(buf, length, PictureType) > 0) {
+    uint8_t PictureType = pes_get_picture_type(buf, length);
 #ifdef LOG_TRICKSPEED
-      if(PictureType != NO_PICTURE) {
-	bool  Video = false, Audio = false;
-	int64_t pts = pes_extract_pts(buf, length, Audio, Video);
-	LOGMSG("    TrickSpeed: frame %s pts %"PRId64, PictureTypeStr(PictureType), pts);
+      if(PictureType != NO_PICTURE && PES_HAS_PTS(buf)) {
+	int64_t pts = pes_get_pts(buf, length);
+	LOGMSG("    TrickSpeed: frame %s pts %"PRId64, picture_type_str[PictureType], pts);
       }
 #endif
-    }
 
-#if 1 
+#if 1
     // limit I-frame rate
     if(PictureType == I_FRAME) {
       static int64_t t0 = 0;
@@ -973,7 +976,7 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
 	
 	t0 += fdelay;
 
-	pes_strip_pts((uchar*)buf, length);
+	pes_change_pts((uchar*)buf, length, INT64_C(0));
       } else {
 	t0 = t1;
       }
@@ -984,14 +987,10 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
   //
   // detecting trick speed mode ?
   //
-  if( m_TrickSpeed > 0 && (m_TrickSpeedMode & trs_PTS_check)) {
-    bool Video = false, Audio = false;
-    int64_t pts = pes_extract_pts(buf, length, Audio, Video);
-
-    if(Video && pts > 0) {
-
-      uchar PictureType = NO_PICTURE;
-      ScanVideoPacket(buf, length, PictureType);
+  if( m_TrickSpeed > 0 && (m_TrickSpeedMode & trs_PTS_check) && IS_VIDEO_PACKET(buf)) {
+    int64_t pts;
+    if (PES_HAS_PTS(buf) && (pts = pes_get_pts(buf, length)) > 0) {
+      uint8_t PictureType = pes_get_picture_type(buf, length);
       if(PictureType != I_FRAME && PictureType != NO_PICTURE) {
 	// --> must be fast worward with IBP frames.
 	// --> PTS check does not work (frames are sent in decoder order) ! */
@@ -1024,15 +1023,15 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
   //
   // Trick speed mode with PTS re-calc
   //
-  if( m_TrickSpeed > 0 && (m_TrickSpeedMode & trs_PTS_recalc)) {
-    bool Video = false, Audio = false;
-    int64_t pts = pes_extract_pts(buf, length, Audio, Video);
-
-    if(Video && pts>0) {
+  if( m_TrickSpeed > 0 && (m_TrickSpeedMode & trs_PTS_recalc) && 
+      IS_VIDEO_PACKET(buf) && PES_HAS_PTS(buf)) {
+    int64_t pts = pes_get_pts(buf, length);
+    if (pts > 0) {
+      
       /* m_TrickSpeedPts could be 0 in case of slow backwards */
-      if(m_TrickSpeedPts == 0) {
+      if(m_TrickSpeedPts == 0)
 	m_TrickSpeedPts = pts;
-      }
+
       LOGTRICKSPEED("    pts %"PRId64" -> %"PRId64" (diff %"PRId64")  %"PRId64"", pts, 
 		    m_TrickSpeedPts + 40*12*90, m_TrickSpeedPts + 40*12*90 - pts,
 		    (m_TrickSpeedPts + 40*12*90)^0x80000000);
@@ -1040,18 +1039,16 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
       pts ^= 0x80000000; /* discontinuity (when mode changes) forces re-syncing of all clocks */
       pes_change_pts((uchar*)buf, length, pts);
     }
-    if(Audio) {
-      LOGMSG("PlayTrickSpeed: got audio frame !");
-    }
   }
 
 #if 1
-  else if(m_TrickSpeedMode & trs_I_frames) {
-    bool Video = false, Audio = false;
-    int64_t pts = pes_extract_pts(buf, length, Audio, Video);
-    if(Video && pts>0) {
-      pts ^= 0x80000000; /* discontinuity (when mode changes) forces re-syncing of all clocks */
-      pes_change_pts((uchar*)buf, length, pts);
+  else if (m_TrickSpeedMode & trs_I_frames) {
+    if (IS_VIDEO_PACKET(buf) && PES_HAS_PTS(buf)) {
+      int64_t pts = pes_get_pts(buf, length);
+      if (pts > 0) {
+	pts ^= 0x80000000; /* discontinuity (when mode changes) forces re-syncing of all clocks */
+	pes_change_pts((uchar*)buf, length, pts);
+      }
     }
   }
 #endif
@@ -1086,9 +1083,8 @@ int cXinelibDevice::PlayAny(const uchar *buf, int length)
     }
   }
 
-  bool isMpeg1 = false;
-
-  int len = pes_packet_len(buf, length, isMpeg1);
+  int isMpeg1 = pes_is_mpeg1(buf);
+  int len = pes_packet_len(buf, length);
   if(len>0 && len != length) 
     LOGMSG("cXinelibDevice::PlayAny: invalid data !");
 
@@ -1097,7 +1093,7 @@ int cXinelibDevice::PlayAny(const uchar *buf, int length)
       return 0; /* wait if data is coming in too fast */
   } else if(m_SkipAudio) {
     /* needed for still images when moving cutting marks */
-    pes_strip_pts((uchar*)buf, length);
+    pes_change_pts((uchar*)buf, length, INT64_C(0));
   }
   m_Cleared = false;
   m_FreeBufs --;
@@ -1135,35 +1131,30 @@ int cXinelibDevice::PlayVideo(const uchar *buf, int length)
 #ifdef START_IFRAME
     // Start with I-frame if stream has video
     // wait for first I-frame
-    uchar pictureType;
-    if( ScanVideoPacket(buf, length, /*0,*/pictureType) > 0 && 
-	pictureType == I_FRAME) {
+    if (pes_get_picture_type(buf, length) == I_FRAME) {
       m_StreamStart = false;
     } else {
       return length;
     }
 #endif
 
-    if(IsFrameH264(buf, length)) {
+    if (!m_h264 && pes_is_frame_h264(buf, length)) {
       LOGMSG("cXinelibDevice::PlayVideo: Detected H.264 video");
-//#warning There are SDTV channels using H.264, so we should really check video size ...
-      ForEach(m_clients, &cXinelibThread::SetHDMode, true);
-      m_StreamStart = false;
+      m_h264 = true;
     }
 
-    int Width, Height;
-    if(GetVideoSize(buf, length, &Width, &Height)) {
+    video_size_t Size;
+    if (pes_get_video_size(buf, length, &Size, m_h264 ? 1:0)) {
       m_StreamStart = false;
+      /*if (m_h264)*/ LOGMSG("Detected video size %dx%d", Size.width, Size.height);
       //LOGDBG("Detected video size %dx%d", Width, Height);
-      ForEach(m_clients, &cXinelibThread::SetHDMode, (Width > 800));
+      ForEach(m_clients, &cXinelibThread::SetHDMode, (Size.width > 800));
     }
   }
 
 #ifdef DEBUG_SWITCHING_TIME
   if(m_statusMonitor->switchtimeOff && m_statusMonitor->switchtimeOn) {
-    uchar pictureType;
-    if( ScanVideoPacket(buf, length, pictureType) > 0 && 
-	pictureType == I_FRAME)
+    if (pes_get_picture_type(buf, length) == I_FRAME)
       m_statusMonitor->IFrame();
   }
 #endif
@@ -1275,13 +1266,11 @@ int cXinelibDevice::PlayAudio(const uchar *buf, int length, uchar Id)
   return PlayAny(buf, length);
 }
 
+#if VDRVERSNUM < 10510
 int cXinelibDevice::PlaySpu(const uchar *buf, int length, uchar Id) 
 {
   TRACEF("cXinelibDevice::PlaySpu");
 
-#ifdef SKIP_DVDSPU
-  return length;
-#else
   if(!buf || length < 6)
     return length;
 
@@ -1301,8 +1290,8 @@ int cXinelibDevice::PlaySpu(const uchar *buf, int length, uchar Id)
   }
 
   return PlayAny(buf, length);
-#endif
 }
+#endif
 
 bool cXinelibDevice::Poll(cPoller &Poller, int TimeoutMs) 
 {
@@ -1515,10 +1504,10 @@ uchar *cXinelibDevice::GrabImage(int &Size, bool Jpeg,
 //   - override cDevice::PlayPesPacket to get DVD SPUs
 //
 
+#if VDRVERSNUM < 10510
 int cXinelibDevice::PlayPesPacket(const uchar *Data, int Length, 
 				  bool VideoOnly)
 {
-#ifndef SKIP_DVDSPU
   switch (Data[3]) {
     case 0xBD: { // private stream 1
       int PayloadOffset = Data[8] + 9;
@@ -1538,18 +1527,18 @@ int cXinelibDevice::PlayPesPacket(const uchar *Data, int Length,
     default:
       ;
   }
-#endif
   return cDevice::PlayPesPacket(Data, Length, VideoOnly);
 }
-
+#endif
 
 //
 // Available DVD SPU tracks
 //
 
+#if VDRVERSNUM < 10515
 bool cXinelibDevice::SetCurrentDvdSpuTrack(int Type, bool Force)
 {
-  if(Type == -1 || 
+  if(Type == ttXSubtitleNone || 
      ( Type >= 0 && 
        Type < 64 &&
        m_DvdSpuTrack[Type].id != 0xffff)) {
@@ -1561,18 +1550,30 @@ bool cXinelibDevice::SetCurrentDvdSpuTrack(int Type, bool Force)
   }
   return false;
 }
+#endif
 
+#if VDRVERSNUM >= 10515
+void cXinelibDevice::SetSubtitleTrackDevice(eTrackType Type)
+{
+  if (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly)
+    ForEach(m_clients, &cXinelibThread::SetSubtitleTrack, Type);
+}
+#endif
+
+#if VDRVERSNUM < 10515
 void cXinelibDevice::ClrAvailableDvdSpuTracks(bool NotifyFrontend)
 {
   for(int i=0; i<64; i++)
     m_DvdSpuTrack[i].id = 0xffff;
-  if(m_CurrentDvdSpuTrack >=0 ) {
-    m_CurrentDvdSpuTrack = -1;
+  if(m_CurrentDvdSpuTrack >= 0 ) {
+    m_CurrentDvdSpuTrack = ttXSubtitleNone;
     if(NotifyFrontend)
-      ForEach(m_clients, &cXinelibThread::SpuStreamChanged, -1);
+      ForEach(m_clients, &cXinelibThread::SpuStreamChanged, m_CurrentDvdSpuTrack);
   }
 }
+#endif
 
+#if VDRVERSNUM < 10515
 int cXinelibDevice::NumDvdSpuTracks(void) const
 { 
   int DvdSpuTracks = 0;
@@ -1581,7 +1582,9 @@ int cXinelibDevice::NumDvdSpuTracks(void) const
       DvdSpuTracks++;
   return DvdSpuTracks; 
 }
+#endif
 
+#if VDRVERSNUM < 10515
 const tTrackId *cXinelibDevice::GetDvdSpuTrack(int Type) const
 {
   if(Type >= 0 && Type < 64 &&
@@ -1589,7 +1592,9 @@ const tTrackId *cXinelibDevice::GetDvdSpuTrack(int Type) const
     return &m_DvdSpuTrack[Type];
   return NULL;
 }
+#endif
 
+#if VDRVERSNUM < 10515
 const char *cXinelibDevice::GetDvdSpuLang(int Type) const
 {
   const tTrackId *track = GetDvdSpuTrack(Type);
@@ -1597,7 +1602,9 @@ const char *cXinelibDevice::GetDvdSpuLang(int Type) const
     return track->language[0] ? track->language : NULL;
   return NULL;
 }
+#endif
 
+#if VDRVERSNUM < 10515
 bool cXinelibDevice::SetAvailableDvdSpuTrack(int Type, const char *lang, bool Current)
 {
   if(Type >= 0 && Type < 64) {
@@ -1613,7 +1620,9 @@ bool cXinelibDevice::SetAvailableDvdSpuTrack(int Type, const char *lang, bool Cu
   }
   return false;
 }
+#endif
 
+#if VDRVERSNUM < 10515
 void cXinelibDevice::EnsureDvdSpuTrack(void)
 {
   if(!m_ForcedDvdSpuTrack &&
@@ -1659,6 +1668,7 @@ void cXinelibDevice::EnsureDvdSpuTrack(void)
     }
   }
 }
+#endif
 
 //
 // Metainfo
@@ -1666,16 +1676,16 @@ void cXinelibDevice::EnsureDvdSpuTrack(void)
 
 const char *cXinelibDevice::GetMetaInfo(eMetainfoType Type)
 {
-  if(Type >= 0 && Type < mi_Count)
-    if ( Type == 0 || Type > 3 ||
-        (Type == 1 && xc.playlist_tracknumber == 1) ||
-        (Type == 2 && xc.playlist_artist == 1) ||
-        (Type == 3 && xc.playlist_album == 1)) {
-       return m_MetaInfo[Type];
-    }
-    else {
-       return "";
+  if(Type >= 0 && Type < mi_Count) {
+    if ((Type == miTitle) || 
+        (Type == miTracknumber && xc.playlist_tracknumber == 1) ||
+        (Type == miArtist      && xc.playlist_artist == 1) ||
+        (Type == miAlbum       && xc.playlist_album == 1) ||
+	(Type > miAlbum)) {
+      return m_MetaInfo[Type];
     }
+    return "";
+  }
 
   LOGMSG("cXinelibDevice::GetMetaInfo: unknown metainfo type");
   return "";
