@@ -4,11 +4,12 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend.c,v 1.43 2008/01/10 23:44:46 phelin Exp $
+ * $Id: frontend.c,v 1.52 2008/03/16 21:40:55 phintuka Exp $
  *
  */
 
 #define __STDC_FORMAT_MACROS
+#define __STDC_CONSTANT_MACROS
 #include <inttypes.h>
 
 #include <stdlib.h>
@@ -35,6 +36,7 @@
 
 #include "tools/pes.h"
 #include "tools/general_remote.h"
+#include "tools/iso639.h"
 
 //#define LOG_CONTROL_MESSAGES
 //#define XINELIBOUTPUT_LOG_KEYS
@@ -86,6 +88,15 @@ void cXinelibThread::KeypressHandler(const char *keymap, const char *key,
   }
 }
 
+#include <vdr/status.h>
+class cFrontendStatusMonitor : public cStatus {
+  private:
+    bool& m_SpuLangAuto;
+  public:
+    cFrontendStatusMonitor(bool& SpuLangAuto) : m_SpuLangAuto(SpuLangAuto) {};
+    virtual void SetSubtitleTrack(int /*Index*/, const char * const */*Tracks*/) { m_SpuLangAuto = false; }
+};
+
 void cXinelibThread::InfoHandler(const char *info)
 {
   char *pmap = strdup(info), *map = pmap, *pt;
@@ -94,14 +105,22 @@ void cXinelibThread::InfoHandler(const char *info)
     *pt = 0;
 
   if(!strncmp(info, "TRACKMAP SPU", 12)) {
+    int CurrentTrack = ttXSubtitleAuto;
     map += 12;
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
     cXinelibDevice::Instance().ClrAvailableDvdSpuTracks(false);
+#endif
     while(*map) {
       bool Current = false;
       while(*map == ' ') map++;
       if(*map == '*') {
 	Current = true;
 	map++;
+	if (*map == '-') {
+	  CurrentTrack = atoi(map);
+	  while (*map && *map != ' ') map++;
+	  continue;
+	}
       }
       if(*map >= '0' && *map <= '9') {
 	int id = atoi(map);
@@ -110,10 +129,25 @@ void cXinelibThread::InfoHandler(const char *info)
 	char *lang = map;
 	while(*map && *map != ' ') map++;
 	if(*map == ' ') { *map = 0; map++; };
-	cXinelibDevice::Instance().SetAvailableDvdSpuTrack(id, *lang ? lang : NULL, Current);
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
+	cXinelibDevice::Instance().SetAvailableDvdSpuTrack(id, iso639_2_to_iso639_1(lang), Current);
+#else
+	cXinelibDevice::Instance().SetAvailableTrack(ttSubtitle, id, id+1, iso639_2_to_iso639_1(lang));
+	if (Current) 
+	  CurrentTrack = id;
+#endif
       }
     }
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
     cXinelibDevice::Instance().EnsureDvdSpuTrack();
+#else
+    if (CurrentTrack == ttXSubtitleAuto)
+      cXinelibDevice::Instance().EnsureSubtitleTrack();
+    else if (CurrentTrack == ttXSubtitleNone)
+      cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttNone, true);
+    else
+      cXinelibDevice::Instance().SetCurrentSubtitleTrack(eTrackType(CurrentTrack+ttSubtitleFirst), true);
+#endif
   }
 
   else if(!strncmp(info, "TRACKMAP AUDIO", 14)) {
@@ -132,7 +166,7 @@ void cXinelibThread::InfoHandler(const char *info)
       char *lang = map;
       while(*map && *map != ' ') map++;
       if(*map == ' ') { *map = 0; map++; };
-      cXinelibDevice::Instance().SetAvailableTrack(ttDolby, id, ttDolby+id, *lang ? lang : NULL);
+      cXinelibDevice::Instance().SetAvailableTrack(ttDolby, id, ttDolby+id, iso639_2_to_iso639_1(lang));
       if(Current) 
 	cXinelibDevice::Instance().SetCurrentAudioTrack((eTrackType)(ttDolby+id));
     }
@@ -175,8 +209,13 @@ void cXinelibThread::InfoHandler(const char *info)
     map += 9;
     while(*map == ' ') map++;
     cXinelibDevice::Instance().SetMetaInfo(miDvdTitleNo, map);
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
     if (*map == '0')  // DVD Menu, set spu track to 0
       cXinelibDevice::Instance().SetCurrentDvdSpuTrack(0);
+#else
+    if (*map == '0')  // DVD Menu, set spu track to 0
+      cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttSubtitleFirst);
+#endif
   }
 
   free(pmap);
@@ -196,6 +235,7 @@ cXinelibThread::cXinelibThread(const char *Description) : cThread(Description)
   m_Frames = 0;
   m_bEndOfStreamReached = false;
   m_bPlayingFile = false;
+  m_StatusMonitor = NULL;
 }
 
 cXinelibThread::~cXinelibThread() 
@@ -205,6 +245,8 @@ cXinelibThread::~cXinelibThread()
   m_bStopThread = true;
   if(Active())
     Cancel();
+  if(m_StatusMonitor)
+    delete m_StatusMonitor;
 }
 
 //
@@ -330,11 +372,22 @@ void cXinelibThread::AudioStreamChanged(bool ac3, int StreamId)
     Xine_Control("AUDIOSTREAM", StreamId);
 }
 
+#if VDRVERSNUM < 10515
 void cXinelibThread::SpuStreamChanged(int StreamId)
 {
   TRACEF("cXinelibThread::SpuStreamChanged");
   Xine_Control("SPUSTREAM", StreamId);
 }
+#else
+void cXinelibThread::SetSubtitleTrack(eTrackType Track)
+{
+  TRACEF("cXinelibThread::SetSubtitleTrack");
+  cString buf = cString::sprintf("SPUSTREAM %d%s", 
+				 Track==ttNone ? ttXSubtitleNone  : (Track - ttSubtitleFirst), 
+				 m_SpuLangAuto ? " auto" : "");
+  Xine_Control(buf);
+}
+#endif
 
 void cXinelibThread::Clear(void)
 {
@@ -396,9 +449,7 @@ int cXinelibThread::Play_PES(const uchar *data, int len)
 int cXinelibThread::Play_Mpeg1_PES(const uchar *data1, int len)
 {
   if(!data1[0] && !data1[1] && data1[2] == 0x01 && len>7 && /* header sync bytes */
-     ( VIDEO_STREAM == (data1[3] & ~VIDEO_STREAM_MASK) ||   /*   video stream */
-       AUDIO_STREAM == (data1[3] & ~AUDIO_STREAM_MASK) ||   /*   audio stream */
-       PRIVATE_STREAM1 == data1[3]) &&                      /*   private stream 1 */
+     ( IS_VIDEO_PACKET(data1) || IS_AUDIO_PACKET(data1)) && /* video / audio / ps1 stream */
      ((data1[6] & 0xC0) != 0x80) &&                         /* really mpeg1 pes */
      (len == ((data1[4]<<8) | data1[5]) + 6)) {             /* whole PES packet and nothing else */
     uchar *data2 = new uchar[len+64];
@@ -556,36 +607,32 @@ bool cXinelibThread::LogoDisplay(void)
 {
   TRACEF("cXinelibThread::LogoDisplay");
   
-  char *path = NULL;
+  cString Path;
+  int     fd = -1;
+
   if(Setup.FileName()) {
-    char *setup_path = strdup(Setup.FileName());
-    char *end = strrchr(setup_path, '/');
+    cString SetupPath = Setup.FileName();
+    char *end = strrchr(SetupPath, '/');
     if(end) {
       *end = 0;
-      asprintf(&path, "%s/plugins/xineliboutput/logo.mpv", setup_path);
+      fd = open(Path=cString::sprintf("%s/plugins/xineliboutput/logo.mpv", *SetupPath), O_RDONLY);
     }
-    free(setup_path);
   }
   
-  int fd = path ? open(path, O_RDONLY) : -1;
-  if(fd<0) {
-    free(path);
-    path = strdup(STARTUP_IMAGE_FILE);
-    fd = open(path, O_RDONLY);
-  }
+  if(fd<0)
+    fd = open(Path=STARTUP_IMAGE_FILE, O_RDONLY);
   
   if(fd >= 0) {
     uint8_t *data = (uint8_t*)malloc(STARTUP_MAX_SIZE);
     int datalen = read(fd, data, STARTUP_MAX_SIZE);
     if(datalen == STARTUP_MAX_SIZE) {
-      LOGMSG("WARNING: custom startup image %s too large", path);
+      LOGMSG("WARNING: custom startup image %s too large", *Path);
     } else if(datalen<=0) {
-      LOGERR("error reading custom startup image %s", path);
+      LOGERR("error reading custom startup image %s", *Path);
     } else {
-      LOGMSG("using custom startup image %s", path);
+      LOGMSG("using custom startup image %s", *Path);
       bool r = Play_Mpeg2_ES(data, datalen, VIDEO_STREAM);
       free(data);
-      free(path);
       for(int i=0; i<5 && !Flush(100); i++)
 	;
       return r;
@@ -593,7 +640,6 @@ bool cXinelibThread::LogoDisplay(void)
     free(data);
     close(fd);
   }
-  free(path);
   
   /* use default image */
   extern const unsigned char v_mpg_vdrlogo[];     // vdrlogo_720x576.c
@@ -698,6 +744,12 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
     Lock();
     m_FileName = FileName;
     m_bPlayingFile = true;
+    m_SpuLangAuto = true;
+#if VDRVERSNUM >= 10515
+    if (m_StatusMonitor)
+      DELETENULL(m_StatusMonitor);
+    m_StatusMonitor = new cFrontendStatusMonitor(m_SpuLangAuto);
+#endif
     Unlock();
   }
 
@@ -707,6 +759,8 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
     Lock();
     m_bPlayingFile = false;
     m_FileName = NULL;
+    if (m_StatusMonitor)
+      DELETENULL(m_StatusMonitor);
     Unlock();
   } else {
     if(xc.extsub_size >= 0)
@@ -805,13 +859,13 @@ int cXinelibThread::ConfigurePostprocessing(const char *name, bool on, const cha
 
 int cXinelibThread::ConfigureVideo(int hue, int saturation, 
 				   int brightness, int contrast,
-				   int overscan)
+				   int overscan, int vo_aspect_ratio)
 {
   char cmd[128];
   Xine_Control("OVERSCAN", overscan);
   snprintf(cmd, sizeof(cmd),
-	   "VIDEO_PROPERTIES %d %d %d %d", 
-	  hue, saturation, brightness, contrast);
+	   "VIDEO_PROPERTIES %d %d %d %d %d", 
+	  hue, saturation, brightness, contrast, vo_aspect_ratio);
   return Xine_Control(cmd);
 }
 
