@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend_svr.c,v 1.48 2008/01/09 08:44:15 phintuka Exp $
+ * $Id: frontend_svr.c,v 1.53 2008/03/16 23:47:25 phintuka Exp $
  *
  */
 
@@ -50,6 +50,9 @@
 #define MAX_OSD_TIMEOUTS  (25*5)      /* max. rate 25 updates/s -> at least 5 seconds */
 #define LOG_OSD_BANDWIDTH (128*1024)  /* log messages if OSD bandwidth > 1 Mbit/s */
 
+#define PLAYFILE_CTRL_TIMEOUT   300   /* ms */
+#define PLAYFILE_TIMEOUT       5000   /* ms */
+
 typedef struct {
   int    Size;
   uchar *Data;
@@ -338,34 +341,13 @@ void cXinelibServer::OsdCmd(void *cmd_gen)
   if(cmd_gen) {
     osd_command_t *cmd = (osd_command_t*)cmd_gen;
     osd_command_t cmdnet;
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-    /* -> network order */
-    memset(&cmdnet, 0, sizeof(osd_command_t));
-    cmdnet.cmd = htonl(cmd->cmd);
-    cmdnet.wnd = htonl(cmd->wnd);
-    cmdnet.pts = htonll(cmd->pts);
-    cmdnet.delay_ms = htonl(cmd->delay_ms);
-    cmdnet.x = htons(cmd->x);
-    cmdnet.y = htons(cmd->y);
-    cmdnet.w = htons(cmd->w);
-    cmdnet.h = htons(cmd->h);
-    cmdnet.datalen = htonl(cmd->datalen);
-    cmdnet.num_rle = htonl(cmd->num_rle);
-    cmdnet.colors  = htonl(cmd->colors);
-    if(cmd->data) {
-      cmdnet.raw_data = (uint8_t*)malloc(cmd->datalen);
-      cmdnet.datalen = htonl( recompress_osd_net(cmdnet.raw_data, cmd->data, cmd->num_rle));
-    } else {
-      cmdnet.data = NULL;
-    }
-    cmdnet.palette = cmd->palette;
-#elif __BYTE_ORDER == __BIG_ENDIAN
     memcpy(&cmdnet, cmd, sizeof(osd_command_t));
-    cmdnet.raw_data = (uint8_t *)malloc(cmd->datalen);
-    cmdnet.datalen = recompress_osd_net(cmdnet.raw_data, cmd->data, cmd->num_rle);
-#else
-#  error __BYTE_ORDER not defined !
-#endif
+    if (cmd->data) {
+      cmdnet.raw_data = (uint8_t *)malloc(cmd->datalen);
+      cmdnet.datalen = recompress_osd_net(cmdnet.raw_data, cmd->data, cmd->num_rle);
+    }
+    // -> network byte order
+    hton_osdcmd(cmdnet);
 
 #ifdef HTTP_OSD
     uint8_t *spudata = NULL;
@@ -766,9 +748,7 @@ int cXinelibServer::PlayFileCtrl(const char *Cmd)
     int64_t t = cTimeMs::Now();
 #endif
 
-    int timeout = 300;
-    if(bPlayfile)
-      timeout = 5000;
+    int timeout = bPlayfile ? PLAYFILE_TIMEOUT : PLAYFILE_CTRL_TIMEOUT;
 
     future.Wait(timeout);
 
@@ -1159,10 +1139,12 @@ void cXinelibServer::Handle_Control_CONFIG(int cli)
   ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay,
 			  xc.audio_compression, xc.audio_equalizer,
                           xc.audio_surround, xc.speaker_type);
-  ConfigureVideo(xc.hue, xc.saturation, xc.brightness, xc.contrast, xc.overscan);
+  ConfigureVideo(xc.hue, xc.saturation, xc.brightness, xc.contrast, xc.overscan, xc.vo_aspect_ratio);
   ConfigurePostprocessing("upmix",     xc.audio_upmix ? true : false, NULL);
   ConfigurePostprocessing("autocrop",  xc.autocrop    ? true : false, 
 			  xc.AutocropOptions());
+  ConfigurePostprocessing("swscale", xc.swscale ? true : false, 
+			  xc.SwScaleOptions());
   ConfigurePostprocessing("pp", xc.ffmpeg_pp ? true : false, 
 			  xc.FfmpegPpOptions());
   ConfigurePostprocessing("unsharp",xc.unsharp ? true : false,
