@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: media_player.c,v 1.43 2008/01/10 23:41:24 phelin Exp $
+ * $Id: media_player.c,v 1.48 2008/03/16 21:45:32 phintuka Exp $
  *
  */
 
@@ -56,6 +56,7 @@ class cXinelibPlayer : public cPlayer
 
     // cPlayer
     virtual void SetAudioTrack(eTrackType Type, const tTrackId *TrackId);
+    virtual void SetSubtitleTrack(eTrackType Type, const tTrackId *TrackId);
     virtual bool GetIndex(int &Current, int &Total, bool SnapToIFrame = false);
     virtual bool GetReplayMode(bool &Play, bool &Forward, int &Speed);
 
@@ -125,6 +126,14 @@ void cXinelibPlayer::SetAudioTrack(eTrackType Type, const tTrackId *TrackId)
     Control("AUDIOSTREAM AC3 %d", (int)(Type - ttAudioFirst));
 }
 
+void cXinelibPlayer::SetSubtitleTrack(eTrackType Type, const tTrackId *TrackId)
+{
+  LOGMSG("cXinelibPlayer::SetSubtitleTrack(%d %s)", (int)Type, TrackId ? TrackId->language : "?");
+#if VDRVERSNUM >= 10515
+  cXinelibDevice::Instance().SetSubtitleTrackDevice(Type);
+#endif
+}
+
 bool cXinelibPlayer::GetIndex(int &Current, int &Total, bool SnapToIFrame) 
 { 
   // Returns the current and total frame index, optionally snapped to the
@@ -655,6 +664,10 @@ eOSState cXinelibPlayerControl::ProcessKey(eKeys Key)
                       m_Player->Control("SEEK 0");    break;
                     }
                     break;
+      case kUser8:
+      case k1:      m_Player->Control("SEEK -20");  break;
+      case kUser9:
+      case k3:      m_Player->Control("SEEK +20");  break;
       case k2:      xc.subtitle_vpos -= 10;
       case k5:      xc.subtitle_vpos += 5;
                     m_Player->Control("SUBTITLES %d", xc.subtitle_vpos);
@@ -887,6 +900,7 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
     if (ti && ti[0] && (!m_CurrentDVDTitle || !strstr(m_CurrentDVDTitle, ti))) {
       memset(m_CurrentDVDTitle, 0, 63);
       strn0cpy(m_CurrentDVDTitle, ti, 63);
+      m_Player->Playlist().Current()->Title = cString::sprintf("%s", m_CurrentDVDTitle);
       MsgReplaying(m_CurrentDVDTitle, NULL);
     }
   }
@@ -969,7 +983,7 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
     case kRed:    Hide();
                   Menu = new cDvdMenu();
 		  break;
-
+#if VDRVERSNUM < 10515
     // SPU channel
     case k5:      cXinelibDevice::Instance().SetCurrentDvdSpuTrack(
                        cXinelibDevice::Instance().GetCurrentDvdSpuTrack() - 2);
@@ -977,7 +991,7 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
                   cRemote::Put(kRed); /* shortcut key */ 
 		  cRemote::Put(k2);
                   break;
-
+#endif
     // Playback control
     case kGreen:  m_Player->Control("SEEK -60");  break;
     case kYellow: m_Player->Control("SEEK +60");  break;
