@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menu.c,v 1.44 2008/01/01 08:25:02 phintuka Exp $
+ * $Id: menu.c,v 1.52 2008/03/16 22:53:23 phintuka Exp $
  *
  */
 
@@ -43,9 +43,9 @@
 				      symlink $(CONFDIR)/plugins/xineliboutput/default_playlist */
 # define HOTKEY_ADELAY_UP    kUp   /* audio delay up */
 # define HOTKEY_ADELAY_DOWN  kDown /* audio delay down */
+# define HOTKEY_TOGGLE_VO_ASPECT kRight
 #endif
 
-//#define OLD_SPU_MENU
 //#define OLD_TOGGLE_FE
 
 #define ISNUMBERKEY(k) (RAWKEY(k) >= k0 && RAWKEY(k) <= k9)
@@ -55,7 +55,7 @@
 class cMenuBrowseFiles : public cOsdMenu 
 {
   protected:
-    eMainMenuMode m_Mode;
+    const eMainMenuMode m_Mode;
     bool          m_OnlyQueue;
     char         *m_CurrentDir;
     char         *m_ConfigLastDir;
@@ -101,10 +101,10 @@ cMenuBrowseFiles::cMenuBrowseFiles(eMainMenuMode mode, bool OnlyQueue) :
     cOsdMenu( ( mode==ShowImages ? tr("Images") :
 		mode==ShowMusic ? (!OnlyQueue ? tr("Play music") : tr("Add to playlist")) :
 		/*mode==ShowFiles ?*/ tr("Play file")),
-	      2, 4)
+	      2, 4),
+    m_Mode(mode)
 {
   m_CurrentDir = NULL;
-  m_Mode       = mode;
   m_OnlyQueue  = OnlyQueue;
 
   m_ConfigLastDir = GetLastDir();
@@ -287,7 +287,10 @@ eOSState cMenuBrowseFiles::Open(bool ForceOpen, bool Parent, bool Queue)
     cString f = cString::sprintf("%s%s/%s", 
 				 GetCurrent()->IsDvd() ? "dvd:" : "",
 				 m_CurrentDir, GetCurrent()->Name());
-    strn0cpy(m_ConfigLastDir, f, sizeof(xc.browse_files_dir));
+    if (GetCurrent()->IsDvd()) 
+      strn0cpy(m_ConfigLastDir, m_CurrentDir, sizeof(xc.browse_files_dir));
+    else
+      strn0cpy(m_ConfigLastDir, f, sizeof(xc.browse_files_dir));
     StoreConfig();
 
     if(m_Mode != ShowImages) {
@@ -469,59 +472,9 @@ eOSState cMenuBrowseFiles::ProcessKey(eKeys Key)
 }
 
 
-//-------------------------- cDvdSpuTrackSelect ------------------------------
-
-#ifdef OLD_SPU_MENU
-class cDvdSpuTrackSelect : public cOsdMenu
-{
-  public:
-    cDvdSpuTrackSelect(void);
-    virtual eOSState ProcessKey(eKeys Key);
-};
-
-cDvdSpuTrackSelect::cDvdSpuTrackSelect(void) : 
-      cOsdMenu(tr("Select subtitle track")) 
-{
-  int count = cXinelibDevice::Instance().NumDvdSpuTracks();
-  int id = 0;
-  int current = cXinelibDevice::Instance().GetCurrentDvdSpuTrack();
-  Add(new cOsdItem("None", osUser1));
-  while(count && id < 64) {
-    const tTrackId *track = cXinelibDevice::Instance().GetDvdSpuTrack(id);
-    if(track) {
-      cString name = "";
-      if(track->language[0])
-	name = cString::sprintf(": %s", track->language);
-      name = cString::sprintf("Track %d%s", id, *name);
-      Add(new cOsdItem(name, osUser1));
-      count--;
-      if(id == current)
-	SetCurrent(Get(Count()-1));
-    }
-    id++;
-  }
-}
-
-eOSState cDvdSpuTrackSelect::ProcessKey(eKeys Key)
-{
-  eOSState state = cOsdMenu::ProcessKey(Key);
-
-  switch(state) {
-    case osUser1: 
-      {
-	const char *txt = Get(Current())->Text();
-	int id = -1; /* -> DVD SPU off */
-	sscanf(txt, "Track %d", &id);
-	cXinelibDevice::Instance().SetCurrentDvdSpuTrack(id);
-	AddSubMenu(new cMenuBrowseFiles());
-	return osEnd;
-      }
-    default: break;
-  }
-  return state;
-}
-#else
+#if VDRVERSNUM < 10515
 
+//-------------------------- cDisplaySpuTracks ------------------------------
 //
 // cDisplaySpuTracks : almost identical copy of VDR 1.4.5 cDisplayTracks
 //
@@ -555,7 +508,7 @@ cDisplaySpuTracks::cDisplaySpuTracks(void) : cOsdObject(true)
   int CurrentTrack = cXinelibDevice::Instance().GetCurrentDvdSpuTrack();
 
   track = numTracks;
-  types[numTracks] = eTrackType(-1);
+  types[numTracks] = eTrackType(ttXSubtitleNone);
   descriptions[numTracks] = strdup("(none)");
   numTracks++;
 
@@ -648,7 +601,7 @@ eOSState cDisplaySpuTracks::ProcessKey(eKeys Key)
      }
   return timeout.TimedOut() ? osEnd : osContinue;
 }
-#endif
+#endif // VDRVERSNUM < 10515
 
 //----------------------------- cMenuXinelib ---------------------------------
 
@@ -692,9 +645,10 @@ cMenuXinelib::cMenuXinelib()
     Add(new cOsdItem(tr("Play remote CD >>"), osUser6));
   else
     Add(new cOsdItem(tr("Play audio CD >>"), osUser6));
+#if VDRVERSNUM < 10515
   if(cXinelibDevice::Instance().NumDvdSpuTracks() > 0)
     Add(new cOsdItem(tr("Select subtitle track >>"), osUser5));
-
+#endif
   Add(NewTitle(tr("Video settings")));
   Add(ctrl_novideo = new cMenuEditBoolItem(tr("Play only audio"), 
 					   &novideo));
@@ -747,7 +701,7 @@ cMenuXinelib::~cMenuXinelib()
 
   if(xc.overscan != overscan)
     cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, xc.brightness,
-					      xc.contrast, xc.overscan);
+					      xc.contrast, xc.overscan, xc.vo_aspect_ratio);
 
   if(xc.headphone != headphone)
     cXinelibDevice::Instance().ConfigurePostprocessing("headphone", 
@@ -807,11 +761,8 @@ eOSState cMenuXinelib::ProcessKey(eKeys Key)
       cControl::Shutdown();
       cControl::Launch(new cXinelibPlayerControl(ShowMusic, "cdda:/"));
       return osEnd;
+#if VDRVERSNUM < 10515
     case osUser5:
-#ifdef OLD_SPU_MENU
-      AddSubMenu(new cDvdSpuTrackSelect());
-      return osContinue;
-#else
       if(!g_PendingMenuAction) {
 	g_PendingMenuAction = cDisplaySpuTracks::Create();
 	return osPlugin;
@@ -836,7 +787,7 @@ eOSState cMenuXinelib::ProcessKey(eKeys Key)
 							 xc.audio_surround, xc.speaker_type);
     else if(item == ctrl_overscan)
       cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, xc.brightness,
-                                                xc.contrast, overscan);
+                                                xc.contrast, overscan, xc.vo_aspect_ratio);
   }
   if(Key==kLeft || Key==kRight) {
     if(item == ctrl_headphone)
@@ -876,7 +827,7 @@ void cMenuXinelib::Store(void)
 eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 {
   eOSState NewState = osEnd;
-  char    *Message  = NULL;
+  cString  Message;
   time_t   now      = time(NULL);
   bool     OnlyInfo = ((g_LastHotkeyTime < now-3) || g_LastHotkey != Key);
 
@@ -891,31 +842,8 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
       cControl::Launch(new cXinelibDvdPlayerControl("dvd:/1"));
       break;
 
+#if VDRVERSNUM < 10515
     case HOTKEY_DVD_SPU:
-#ifdef OLD_SPU_MENU
-      {
-	int count = cXinelibDevice::Instance().NumDvdSpuTracks();
-	int current = cXinelibDevice::Instance().GetCurrentDvdSpuTrack();
-	if(!OnlyInfo) {
-	  current++;
-	  if(current == count)
-	    current = -1;
-	  cXinelibDevice::Instance().SetCurrentDvdSpuTrack(current);
-	}
-	const char *lang = cXinelibDevice::Instance().GetDvdSpuLang(current); 
-	if(current == -1) lang = "default";
-	if(count<1)
-	  asprintf(&Message, "%s", tr("No subtitles available!"));
-	if(lang && lang[0])
-	  asprintf(&Message, "%s %s %s (%d)", tr("Subtitles"), 
-		   OnlyInfo ? ":" : "->",
-		   lang, current);
-	else
-	  asprintf(&Message, "%s %s %d", tr("Subtitles"), 
-		   OnlyInfo ? ":" : "->", 
-		   current);
-      }
-#else
       /* use audio track display menu */
       if(!g_PendingMenuAction) {
 	bool WasOpen = cDisplaySpuTracks::IsOpen();
@@ -924,12 +852,11 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 	  cRemote::CallPlugin("xineliboutput");
 	  if(WasOpen || !OnlyInfo) cRemote::Put(kNext);
 	} else {
-	  asprintf(&Message, "%s", tr("No subtitles available!"));
+	  Message = tr("No subtitles available!");
 	}
       }
-#endif
       break;
-
+#endif
     case HOTKEY_LOCAL_FE:
       /* off, on */
       {
@@ -966,9 +893,9 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 	      xc.fullscreen, xc.width, xc.height, xc.modeswitch, xc.modeline, 
 	      xc.display_aspect, xc.scale_video, xc.field_order);
 	}
-	asprintf(&Message, "%s %s %s", tr("Local Frontend"), 
-		 OnlyInfo ? ":" : "->", 
-		 xc.s_frontendNames[local_frontend]);
+	Message = cString::sprintf("%s %s %s", tr("Local Frontend"), 
+				   OnlyInfo ? ":" : "->", 
+				   xc.s_frontendNames[local_frontend]);
       }
       break;
 
@@ -980,9 +907,21 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 						   xc.modeswitch, xc.modeline, xc.display_aspect, 
 						   xc.scale_video, xc.field_order);      
       }
-      asprintf(&Message, "%s %s %s", tr("Aspect ratio"), 
-	       OnlyInfo ? ":" : "->",
-	       tr(xc.s_aspects[xc.display_aspect]));
+      Message = cString::sprintf("%s %s %s", tr("Aspect ratio"), 
+				 OnlyInfo ? ":" : "->",
+				 tr(xc.s_aspects[xc.display_aspect]));
+      break;
+
+    case HOTKEY_TOGGLE_VO_ASPECT:
+      /* auto, square, 4:3, anamorphic or DVB */
+      if(!OnlyInfo) {
+        xc.vo_aspect_ratio = (xc.vo_aspect_ratio < VO_ASPECT_count-1) ? xc.vo_aspect_ratio + 1 : 0;
+        cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, xc.brightness,
+                                              xc.contrast, xc.overscan, xc.vo_aspect_ratio);
+      }
+      Message = cString::sprintf("%s %s %s", tr("Video aspect ratio"),
+				 OnlyInfo ? ":" : "->",
+				 tr(xc.s_vo_aspects[xc.vo_aspect_ratio]));
       break;
 
     case HOTKEY_TOGGLE_CROP:    
@@ -1001,9 +940,9 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 							   xc.AutocropOptions());
       }
 
-      asprintf(&Message, "%s %s %s", tr("Crop letterbox 4:3 to 16:9"), 
-	       OnlyInfo ? ":" : "->",
-	       !xc.autocrop ? tr("Off") : xc.autocrop_autodetect ? tr("automatic") : tr("On"));
+      Message = cString::sprintf("%s %s %s", tr("Crop letterbox 4:3 to 16:9"), 
+				 OnlyInfo ? ":" : "->",
+				 !xc.autocrop ? tr("Off") : xc.autocrop_autodetect ? tr("automatic") : tr("On"));
       break;
 
     case HOTKEY_DEINTERLACE:    
@@ -1020,9 +959,9 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 							     compression, xc.audio_equalizer, 
 							     xc.audio_surround, xc.speaker_type);
 	}
-	asprintf(&Message, "%s %s %s", tr("Deinterlacing"), 
-		 OnlyInfo ? ":" : "->", 
-		 tr(off ? "Off":"On"));
+	Message = cString::sprintf("%s %s %s", tr("Deinterlacing"), 
+				   OnlyInfo ? ":" : "->", 
+				   tr(off ? "Off":"On"));
       }
       break;
 
@@ -1033,10 +972,10 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 	cXinelibDevice::Instance().ConfigurePostprocessing(
 		  "upmix", xc.audio_upmix ? true : false, NULL);
       }
-      asprintf(&Message, "%s %s %s", 
-	       tr("Upmix stereo to 5.1"), 
-	       OnlyInfo ? ":" : "->",
-	       tr(xc.audio_upmix ? "On" : "Off"));
+      Message = cString::sprintf("%s %s %s", 
+				 tr("Upmix stereo to 5.1"), 
+				 OnlyInfo ? ":" : "->",
+				 tr(xc.audio_upmix ? "On" : "Off"));
       break;
 
     case HOTKEY_DOWNMIX:    
@@ -1047,10 +986,10 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 	    xc.deinterlace_method, xc.audio_delay, xc.audio_compression, 
 	    xc.audio_equalizer, xc.audio_surround, xc.speaker_type);
       }
-      asprintf(&Message, "%s %s %s", 
-	       tr("Downmix AC3 to surround"), 
-	       OnlyInfo ? ":" : "->",
-	       tr(xc.audio_surround ? "On":"Off"));
+      Message = cString::sprintf("%s %s %s", 
+				 tr("Downmix AC3 to surround"), 
+				 OnlyInfo ? ":" : "->",
+				 tr(xc.audio_surround ? "On":"Off"));
       break;
 
       case HOTKEY_PLAYLIST:
@@ -1064,7 +1003,7 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 	    if (S_ISLNK(st.st_mode)) {
 	      char *buffer = ReadLink(file);
 	      if (!buffer || stat(buffer, &st)) {
-		asprintf(&Message, tr("Default playlist not found"));
+		Message = tr("Default playlist not found");
 	      } else {
 		LOGDBG("Replaying default playlist: %s", file);
 		cControl::Shutdown();
@@ -1072,10 +1011,10 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
 	      }
 	      free(buffer);
 	    } else {
-	      asprintf(&Message, tr("Default playlist is not symlink"));
+	      Message = tr("Default playlist is not symlink");
 	    }
 	  } else {
-	    asprintf(&Message, tr("Default playlist not defined"));
+	    Message = tr("Default playlist not defined");
 	  }
 	  free(file);
 	}
@@ -1089,9 +1028,9 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
                                                            xc.audio_compression, xc.audio_equalizer,
                                                            xc.audio_surround, xc.speaker_type);
       }
-      asprintf(&Message, "%s %s %d %s", tr("Delay"),
-	       OnlyInfo ? ":" : "->",
-	       xc.audio_delay, tr("ms"));
+      Message = cString::sprintf("%s %s %d %s", tr("Delay"),
+				 OnlyInfo ? ":" : "->",
+				 xc.audio_delay, tr("ms"));
       break;
 
     case HOTKEY_ADELAY_DOWN:
@@ -1102,22 +1041,21 @@ eOSState cMenuXinelib::ProcessHotkey(eKeys Key)
                                                            xc.audio_compression, xc.audio_equalizer,
                                                            xc.audio_surround, xc.speaker_type);
       }
-      asprintf(&Message, "%s %s %d %s", tr("Delay"),
-	       OnlyInfo ? ":" : "->",
-	       xc.audio_delay, tr("ms"));
+      Message = cString::sprintf("%s %s %d %s", tr("Delay"),
+				 OnlyInfo ? ":" : "->",
+				 xc.audio_delay, tr("ms"));
       break;
 
     default:
-      asprintf(&Message, tr("xineliboutput: hotkey %s not binded"), cKey::ToString(Key));
+      Message = cString::sprintf(tr("xineliboutput: hotkey %s not binded"), cKey::ToString(Key));
       break;
   }
 
-  if(Message) {
+  if(*Message) {
     if(!g_PendingMenuAction &&
        !cRemote::HasKeys() &&
        cRemote::CallPlugin("xineliboutput"))
       g_PendingMenuAction = new cDisplayMessage(Message);
-    free(Message);
   }
 
   g_LastHotkeyTime = now;
