@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: setup_menu.c,v 1.42 2008/01/10 23:36:06 phelin Exp $
+ * $Id: setup_menu.c,v 1.48 2008/03/13 22:19:33 phintuka Exp $
  *
  */
 
@@ -380,7 +380,7 @@ void cMenuSetupAudioEq::Store(void)
 
 //--- cMenuSetupVideo --------------------------------------------------------
 
-static const char *tvtime_method[] =
+static const char * const tvtime_method[] =
   { "use_vo_driver",
     "Linear",
     "LinearBlend",
@@ -394,9 +394,9 @@ static const char *tvtime_method[] =
     "TomsMoComp",
     NULL};
 
-static const int tvtime_methods_count = (sizeof(tvtime_method)-1)/sizeof(char*);
+static const int tvtime_methods_count = (sizeof(tvtime_method)/sizeof(tvtime_method[0]) - 1);
 
-static const char *tvtime_method_name[] =
+static const char * const tvtime_method_name[] =
   {"Use Video-Out Driver",               // "use_vo_driver"
    "Linear Interpolation",               // "Linear",
    "Linear Blend (mplayer)",             // "LinearBlend",
@@ -410,23 +410,23 @@ static const char *tvtime_method_name[] =
    "Tom's Motion Compensated (DScaler)", // "TomsMoComp",
    NULL};
 
-static const char *tvtime_pulldown[] = 
+static const char * const tvtime_pulldown[] = 
   { "none",
     "vector",
     NULL};
 
-static const char *tvtime_pulldown_name[] = 
+static const char * const tvtime_pulldown_name[] = 
   { "None",
     "Vector",
     NULL};
 
-static const char *tvtime_framerate[] = 
+static const char * const tvtime_framerate[] = 
   { "full",
     "half_top",
     "half_bottom",
     NULL};
 
-static const char *tvtime_framerate_name[] = 
+static const char * const tvtime_framerate_name[] = 
   { "Full",
     "Half (top)",
     "Half (bottom)",
@@ -485,6 +485,11 @@ class cMenuSetupVideo : public cMenuSetupPage
     config_t newconfig;
 
     cOsdItem *ctrl_autocrop;
+    cOsdItem *ctrl_swscale;
+    cOsdItem *ctrl_swscale_resize;
+    cOsdItem *ctrl_swscale_aspect;
+    cOsdItem *ctrl_swscale_width;
+    cOsdItem *ctrl_swscale_height;
     cOsdItem *ctrl_hue;
     cOsdItem *ctrl_saturation;
     cOsdItem *ctrl_contrast;
@@ -495,6 +500,7 @@ class cMenuSetupVideo : public cMenuSetupPage
     cOsdItem *ctrl_tvtime_method;
     cOsdItem *ctrl_unsharp;
     cOsdItem *ctrl_denoise3d;
+    cOsdItem *ctrl_vo_aspect_ratio;
 
     int deinterlace;
     struct tvtime_s tvtime;
@@ -530,9 +536,11 @@ cMenuSetupVideo::~cMenuSetupVideo(void)
 {
   cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, 
 					    xc.brightness, xc.contrast,
-					    xc.overscan);
+					    xc.overscan, xc.vo_aspect_ratio);
   cXinelibDevice::Instance().ConfigurePostprocessing(
        "autocrop", xc.autocrop ? true : false, xc.AutocropOptions());
+  cXinelibDevice::Instance().ConfigurePostprocessing(
+       "swscale", xc.swscale ? true : false, xc.SwScaleOptions());
   cXinelibDevice::Instance().ConfigurePostprocessing(
        "pp", xc.ffmpeg_pp ? true : false, xc.FfmpegPpOptions());
   cXinelibDevice::Instance().ConfigurePostprocessing(
@@ -552,6 +560,10 @@ void cMenuSetupVideo::Set(void)
 
   Add(NewTitle(tr("Video")));
 
+  Add(ctrl_vo_aspect_ratio =
+      new cMenuEditStraI18nItem(tr("Aspect ratio"), &newconfig.vo_aspect_ratio,
+                                  VO_ASPECT_count, xc.s_vo_aspects));
+
   Add(ctrl_autocrop = 
       new cMenuEditBoolItem(tr("Crop letterbox 4:3 to 16:9"), 
 			    &newconfig.autocrop));
@@ -567,6 +579,29 @@ void cMenuSetupVideo::Set(void)
 			      &newconfig.autocrop_subs));
   }
 
+  ctrl_swscale_resize = ctrl_swscale_aspect = ctrl_swscale_width = ctrl_swscale_height = NULL;
+  Add(ctrl_swscale =
+      new cMenuEditBoolItem(tr("Software scaling"), 
+			    &newconfig.swscale));
+  if(newconfig.swscale) {
+    Add(ctrl_swscale_aspect =
+	new cMenuEditBoolItem(tr("  Change aspect ratio"), 
+			      &newconfig.swscale_change_aspect));
+    Add(ctrl_swscale_resize = 
+	new cMenuEditBoolItem(tr("  Change video size"), 
+			      &newconfig.swscale_resize));
+    if(newconfig.swscale_resize) {
+      Add(ctrl_swscale_width =
+	  new cMenuEditIntItem( tr("  Width"), 
+				&newconfig.swscale_width, 360, 2000));
+      Add(ctrl_swscale_height =
+	  new cMenuEditIntItem( tr("  Height"),
+				&newconfig.swscale_height, 288, 1200));
+      Add(new cMenuEditBoolItem(tr("  Allow downscaling"),
+				&newconfig.swscale_downscale));
+    }
+  }
+
   Add(ctrl_overscan =
       new cMenuEditTypedIntItem(tr("Overscan (crop image borders)"), "%", 
 				&newconfig.overscan, 0, 10,
@@ -679,20 +714,21 @@ eOSState cMenuSetupVideo::ProcessKey(eKeys Key)
 
   if(item == ctrl_hue || item == ctrl_saturation || 
      item == ctrl_contrast || item == ctrl_brightness ||
-     item == ctrl_overscan)
+     item == ctrl_overscan || item == ctrl_vo_aspect_ratio)
 #ifdef INTEGER_CONFIG_VIDEO_CONTROLS
     cXinelibDevice::Instance().ConfigureVideo(newconfig.hue, 
 					      newconfig.saturation,
 					      newconfig.brightness, 
 					      newconfig.contrast,
-					      newconfig.overscan);
+					      newconfig.overscan,
+                                              newconfig.vo_aspect_ratio);
 #else
     cXinelibDevice::Instance().ConfigureVideo(
        INDEX_TO_CONTROL(newconfig.hue), 
        INDEX_TO_CONTROL(newconfig.saturation),
        INDEX_TO_CONTROL(newconfig.brightness), 
        INDEX_TO_CONTROL(newconfig.contrast),
-       newconfig.overscan);
+       newconfig.overscan, newconfig.vo_aspect_ratio);
 #endif
   else if(item == ctrl_autocrop) {
     cXinelibDevice::Instance().ConfigurePostprocessing(
@@ -700,6 +736,16 @@ eOSState cMenuSetupVideo::ProcessKey(eKeys Key)
 	 newconfig.AutocropOptions());
     Set();
   }
+  else if(item == ctrl_swscale || 
+	  item == ctrl_swscale_resize ||
+	  item == ctrl_swscale_aspect ||
+	  item == ctrl_swscale_width ||
+	  item == ctrl_swscale_height) {
+    cXinelibDevice::Instance().ConfigurePostprocessing(
+         "swscale", newconfig.swscale ? true : false, 
+	 newconfig.SwScaleOptions());
+    Set();
+  }
   else if(item == ctrl_pp) {
     cXinelibDevice::Instance().ConfigurePostprocessing(
 	 "pp", newconfig.ffmpeg_pp ? true : false, 
@@ -751,6 +797,12 @@ void cMenuSetupVideo::Store(void)
   SetupStore("Video.AutoCrop.SoftStart",  xc.autocrop_soft);
   SetupStore("Video.AutoCrop.FixedSize",  xc.autocrop_fixedsize);
   SetupStore("Video.AutoCrop.DetectSubs", xc.autocrop_subs);
+  SetupStore("Video.SwScale",           xc.swscale);
+  SetupStore("Video.SwScale.Aspect",    xc.swscale_change_aspect);
+  SetupStore("Video.SwScale.Resize",    xc.swscale_resize);
+  SetupStore("Video.SwScale.Width",     xc.swscale_width);
+  SetupStore("Video.SwScale.Height",    xc.swscale_height);
+  SetupStore("Video.SwScale.Downscale", xc.swscale_downscale);
   SetupStore("Video.HUE",        xc.hue);
   SetupStore("Video.Saturation", xc.saturation);
   SetupStore("Video.Contrast",   xc.contrast);
@@ -758,6 +810,7 @@ void cMenuSetupVideo::Store(void)
   SetupStore("Video.Overscan",   xc.overscan);
   SetupStore("Video.IBPTrickSpeed", xc.ibp_trickspeed);
   SetupStore("Video.MaxTrickSpeed", xc.max_trickspeed);
+  SetupStore("Video.AspectRatio", xc.vo_aspect_ratio);
   SetupStore("Post.pp.Enable",   xc.ffmpeg_pp);
   SetupStore("Post.pp.Quality",  xc.ffmpeg_pp_quality);
   SetupStore("Post.pp.Mode",     xc.ffmpeg_pp_mode);
@@ -792,8 +845,10 @@ class cMenuSetupOSD : public cMenuSetupPage
     cOsdItem *ctrl_scale;
     cOsdItem *ctrl_downscale;
     cOsdItem *ctrl_lowres;
+#if VDRVERSNUM < 10515
     cOsdItem *ctrl_spulang0;
-  
+#endif
+
   protected:
     virtual void Store(void);
     void Set(void);
@@ -835,7 +890,9 @@ void cMenuSetupOSD::Set(void)
   ctrl_lowres = NULL;
   ctrl_alpha = NULL;
   ctrl_alpha_abs = NULL;
+#if VDRVERSNUM < 10515
   ctrl_spulang0 = NULL;
+#endif
 
   Add(NewTitle(tr("On-Screen Display")));
   Add(new cMenuEditBoolItem(tr("Hide main menu"), 
@@ -871,6 +928,7 @@ void cMenuSetupOSD::Set(void)
 				&newconfig.alpha_correction_abs, -0xff, 0xff,
 				tr("Off")));
   
+#if VDRVERSNUM < 10515
   Add(NewTitle(tr("Subtitles")));
   Add(new cMenuEditBoolItem(trVDR("Setup.EPG$Preferred languages"), 
 			    &newconfig.spu_autoshow));
@@ -885,6 +943,7 @@ void cMenuSetupOSD::Set(void)
     Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
 			     newconfig.spu_lang[3], 4, LangNameChars));
   }
+#endif
 
   Add(new cMenuEditStraI18nItem(tr("External subtitle size"),
 				&newconfig.extsub_size, SUBTITLESIZE_count, xc.s_subtitleSizes));
@@ -922,11 +981,12 @@ eOSState cMenuSetupOSD::ProcessKey(eKeys Key)
     Set();
   if(newconfig.unscaled_osd && ctrl_lowres)
     Set();
+#if VDRVERSNUM < 10515
   if(newconfig.spu_autoshow && !ctrl_spulang0)
     Set();
   if(!newconfig.spu_autoshow && ctrl_spulang0)
     Set();
-
+#endif
   return state;
 }
 
@@ -953,11 +1013,13 @@ void cMenuSetupOSD::Store(void)
   SetupStore("OSD.AlphaCorrectionAbs", xc.alpha_correction_abs);
 
   SetupStore("OSD.ExtSubSize", xc.extsub_size);
+#if VDRVERSNUM < 10515
   SetupStore("OSD.SpuAutoSelect", xc.spu_autoshow);
   SetupStore("OSD.SpuLang0", xc.spu_lang[0]);
   SetupStore("OSD.SpuLang1", xc.spu_lang[1]);
   SetupStore("OSD.SpuLang2", xc.spu_lang[2]);
   SetupStore("OSD.SpuLang3", xc.spu_lang[3]);
+#endif
   Setup.Save();
 }
 
@@ -1682,7 +1744,7 @@ eOSState cTestGrayscale::ProcessKey(eKeys key)
 	br -= 0xffff/1024;
 	sprintf(s, "b %d", br);
 	m_Osd->DrawText(400, 100, s, 0xff000000, 0xffffffff, cFont::GetFont(fontSml));
-	cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, br, co, xc.overscan);
+	cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, br, co, xc.overscan, xc.vo_aspect_ratio);
 	m_Osd->Flush();
 	return osContinue;	
       case kUp:
@@ -1691,7 +1753,7 @@ eOSState cTestGrayscale::ProcessKey(eKeys key)
 	co -= 0xffff/1024;
 	sprintf(s, "c %d", co);
 	m_Osd->DrawText(400, 130, s, 0xff000000, 0xffffffff, cFont::GetFont(fontSml));
-	cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, br, co, xc.overscan);
+	cXinelibDevice::Instance().ConfigureVideo(xc.hue, xc.saturation, br, co, xc.overscan, xc.vo_aspect_ratio);
 	m_Osd->Flush();
 	return osContinue;
     }
