@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: playlist.c,v 1.12 2008/01/10 23:37:36 phelin Exp $
+ * $Id: playlist.c,v 1.16 2008/02/19 04:51:18 phintuka Exp $
  *
  */
 
@@ -98,14 +98,13 @@ int cPlaylistItem::Compare(const cListObject &ListObject) const
 // cID3Scanner
 //
 
-static const char *shell_escape(const cString& src, char ch)
+static const char *shell_escape(char *buf, int buflen, const cString& src, char ch)
 {
-  static char buf[4096];
   const char *pt = *src;
   int n = 0;
 
   if(pt) {
-    while(*pt && n < (int)(sizeof(buf)-2)) {
+    while(*pt && n < buflen-2) {
       if(*pt == ch || *pt == '\\' /*|| *pt == '\"' || *pt == '\''*/) {
         buf[n++] = '\\';
       }
@@ -122,9 +121,8 @@ class cID3Scanner : public cThread
  public:
   cPlaylist& m_List;
   cID3Scanner(cPlaylist& List) : cThread("Metadata scanner"), m_List(List), m_Done(false) {};
-  cCondWait wait;
   
-  void CancelScanner(void) { wait.Signal(); Cancel(3); }
+  void CancelScanner(void) { Cancel(3); }
 
  private:
   bool m_Done;
@@ -132,17 +130,27 @@ class cID3Scanner : public cThread
   virtual void Action(void)
   {
     cPlaylistItem *Item = NULL;
+    unsigned int Version = 0;
 
     (void)nice(10);
-    wait.Wait(5000);
 
     LOGDBG("ID3Scanner Started");
     while(Running()) {
+
+      cMutexLock ml(&m_List.m_Lock);
+
+      if(Version < m_List.m_Version) {
+	// restart after sort, add, del
+	Item = NULL;
+	Version = m_List.m_Version;
+      }
+
       if(!(Item = m_List.Next(Item)))
         break;
 
       if(xc.IsAudioFile(Item->Filename)) {
         LOGDBG("Scanning metainfo for file %s", *Item->Filename);
+        char buf[4096];
         cString Cmd = "";
         if(!strcasecmp((Item->Filename) + strlen(Item->Filename) - 5, ".flac"))
             Cmd = cString::sprintf("metaflac "
@@ -151,7 +159,7 @@ class cID3Scanner : public cThread
                                    " --show-tag=ARTIST "
                                    " --show-tag=TRACKNUMBER "
                                    " \"%s\"",
-                                   shell_escape(Item->Filename, '\"'));
+                                   shell_escape(buf, sizeof(buf)-1, Item->Filename, '\"'));
         else
             Cmd = cString::sprintf("mp3info -p \""
                                    "ARTIST=%%a\\r\\n"
@@ -159,11 +167,10 @@ class cID3Scanner : public cThread
                                    "TITLE=%%t\\r\\n"
                                    "TRACKNUMBER=%%n\\r\\n\""
                                    " \"%s\"",
-                                   shell_escape(Item->Filename, '\"'));
+                                   shell_escape(buf, sizeof(buf)-1, Item->Filename, '\"'));
 
         cPipe p;
         if(p.Open(*Cmd, "r")) {
-          cMutexLock ml(&m_List.m_Lock);
           cReadLine r;
           char *pt;
           while(NULL != (pt = r.Read(p))) {
@@ -335,6 +342,7 @@ cPlaylist::cPlaylist()
   m_Menu    = NULL;
   m_Scanner = NULL;
   m_Current = NULL;
+  m_Version = 1;
 }
 
 cPlaylist::~cPlaylist()
@@ -367,6 +375,7 @@ void cPlaylist::Sort(void)
 {
   cMutexLock ml(&m_Lock);
   cListBase::Sort(); 
+  m_Version++;
 }
 
 int cPlaylist::Count(void) const 
@@ -398,6 +407,7 @@ void cPlaylist::Del(cPlaylistItem *it)
 		cList<cPlaylistItem>::Prev(Current());
 
   cListBase::Del(it);
+  m_Version++;
 }
 
 void cPlaylist::SetCurrent(cPlaylistItem *current) 
@@ -863,7 +873,8 @@ bool cPlaylist::Read(const char *PlaylistFile, bool Recursive)
     LOGMSG("Empty playlist %s !", PlaylistFile);
     Add(new cPlaylistItem(PlaylistFile));
   }
-    
+
+  m_Version++;
   return Result;
 }
 
@@ -932,7 +943,7 @@ cString cPlaylist::GetEntry(cPlaylistItem *i, bool isPlaylist, bool isCurrent)
 {
 
   cString Entry = "";
-  if (*i->Artist && xc.playlist_artist || *i->Album && xc.playlist_album) {
+  if ((*i->Artist && xc.playlist_artist) || (*i->Album && xc.playlist_album)) {
       Entry = cString::sprintf("%s%s%s%s%s%s(%s%s%s)",
                                isPlaylist ? (isCurrent ? "*" : " ") : "",
                                isPlaylist ? "\t" : " ",
