@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend.c,v 1.47 2008/01/03 20:06:27 phintuka Exp $
+ * $Id: xine_frontend.c,v 1.51 2008/03/13 23:32:16 phintuka Exp $
  *
  */
 
@@ -609,13 +609,11 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
 
   if(audio_driver && !strcmp(audio_driver, "auto")) {
     this->audio_port = xine_open_audio_driver (this->xine, NULL, NULL);
-  } else if(audio_driver && !strcmp(audio_driver, "none")) {
 #if XINE_VERSION_CODE < 10190
+  } else if(audio_driver && !strcmp(audio_driver, "none")) {
     this->audio_port = _x_ao_new_port (this->xine, NULL, 1);
-#else
-    this->audio_port = xine_new_framegrab_audio_port(this->xine);
-#endif
     this->audio_port->set_property(this->audio_port, AO_PROP_DISCARD_BUFFERS, 1);
+#endif
   } else {
     this->audio_port = xine_open_audio_driver (this->xine, audio_driver, NULL);
   }
@@ -787,7 +785,7 @@ static void init_dummy_ports(fe_t *this, int on)
 #if XINE_VERSION_CODE < 10190
       this->audio_port_none = _x_ao_new_port (this->xine, NULL, 1); 
 #else
-      this->audio_port_none = xine_new_framegrab_audio_port(this->xine);
+    this->audio_port_none = NULL;/*xine_new_framegrab_audio_port(this->xine);*/
 #endif
     if(this->audio_port_none)
       this->audio_port_none->set_property(this->audio_port_none, AO_PROP_DISCARD_BUFFERS, 1);
@@ -968,6 +966,26 @@ static void fe_post_open(const fe_t *this, const char *name, const char *args)
   } else
     strn0cpy(initstr, name, sizeof(initstr));
 
+  /* swscale aspect ratio */
+  if (!strcmp(name, "swscale")) {
+    char *pt = strstr(initstr, "output_aspect=auto");
+    if (pt) {
+      char tmp[16];
+      double r = 0.0;
+      pt += 14;
+      switch(this->aspect) {
+      case 0:
+      case 1: /*       */ r = this->display_ratio * (double)this->width / (double)this->height; break;
+      case 2: /* 4:3   */ r = 4.0/3.0; break;
+      case 3: /* 16:9  */ r = 16.0/9.0; break;
+      case 4: /* 16:10 */ r = 16.0/10.0; break;
+      }
+      /* in finnish locale decimal separator is "," - same as post plugin parameter separator :( */
+      sprintf(tmp, "%04d", (int)(r*1000.0));
+      strncpy(pt, tmp, 4); 
+    }
+  }
+
   LOGDBG("opening post plugin: %s", initstr);
 
   /* close old audio visualization plugin */
@@ -1414,6 +1432,11 @@ static void *fe_control(void *fe_handle, const char *cmd)
       /* trigger forced redraw to make changes effective */
       xine_set_param(this->stream, XINE_PARAM_VO_ZOOM_X, 100);      
     }
+  } else if(!strncmp(cmd, "VO_ASPECT ", 10)) {
+    int vo_aspect_ratio;
+    if(1 == sscanf(cmd+10, "%d", &vo_aspect_ratio)) {
+      xine_set_param(this->stream, XINE_PARAM_VO_ASPECT_RATIO, vo_aspect_ratio);
+    }
   }
   
 
