@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend_main.c,v 1.35 2008/01/09 20:40:17 phintuka Exp $
+ * $Id: xine_frontend_main.c,v 1.37 2008/02/19 02:11:31 phintuka Exp $
  *
  */
 
@@ -177,8 +177,10 @@ static void *kbd_receiver_thread(void *fe)
   }
 
   do {
+    alarm(0);
     errno = 0;
     code = read_key_seq();
+    alarm(3); /* watchdog */
     if(code == 0)
       continue;
     if(code == 27) {
@@ -208,6 +210,7 @@ static void *kbd_receiver_thread(void *fe)
 
   } while(!terminate_key_pressed && code != 0xffff);
   
+  alarm(0);
   LOGDBG("Keyboard thread terminated");
   tcsetattr(STDIN_FILENO, TCSANOW, &saved_tm);
   system("setterm -cursor on");
@@ -304,7 +307,7 @@ static char *strcatrealloc(char *dest, const char *src)
   return dest;
 }
 
-static const char *help_str = 
+static const char help_str[] = 
 "When server address is not given, server is searched from local network.\n"
 "If server is not found, localhost (127.0.0.1) is used as default.\n\n"
     "   --help                        Show (this) help message\n"
@@ -343,6 +346,8 @@ static const char *help_str =
     "                                 are tried in following order:\n"
     "                                 local pipe, rtp, udp, tcp\n\n";
 
+static const char short_options[] = "HL:A:V:d:a:fw:h:P:vslkbtur";
+
 static const struct option long_options[] = {
   { "help",       no_argument,       NULL, 'H' },
   { "audio",      required_argument, NULL, 'A' },
@@ -405,7 +410,7 @@ int main(int argc, char *argv[])
 	 xmajor, xminor, xsub);
 
   /* Parse arguments */
-  while ((c = getopt_long(argc, argv, "HL:A:V:d:a:fw:h:P:vslkbtur", long_options, NULL)) != -1) {
+  while ((c = getopt_long(argc, argv, short_options, long_options, NULL)) != -1) {
     switch (c) {
     default:
     case 'H': printf("\nUsage: %s [options] [xvdr[+udp|+tcp|+rtp]:[//host[:port]]] \n"
