@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c 1.158 2007/11/25 13:46:27 kls Exp $
+ * $Id: config.c 1.161 2008/02/17 13:39:00 kls Exp $
  */
 
 #include "config.h"
@@ -62,10 +62,10 @@ const char *cCommand::Execute(const char *Parameters)
 {
   free(result);
   result = NULL;
-  char *cmdbuf = NULL;
+  cString cmdbuf;
   if (Parameters)
-     asprintf(&cmdbuf, "%s %s", command, Parameters);
-  const char *cmd = cmdbuf ? cmdbuf : command;
+     cmdbuf = cString::sprintf("%s %s", command, Parameters);
+  const char *cmd = *cmdbuf ? *cmdbuf : command;
   dsyslog("executing command '%s'", cmd);
   cPipe p;
   if (p.Open(cmd, "r")) {
@@ -82,11 +82,10 @@ const char *cCommand::Execute(const char *Parameters)
      }
   else
      esyslog("ERROR: can't open pipe for command '%s'", cmd);
-  free(cmdbuf);
   return result;
 }
 
-// -- cSVDRPhost -------------------------------------------------------------
+// --- cSVDRPhost ------------------------------------------------------------
 
 cSVDRPhost::cSVDRPhost(void)
 {
@@ -122,12 +121,12 @@ bool cSVDRPhost::Accepts(in_addr_t Address)
   return (Address & mask) == (addr.s_addr & mask);
 }
 
-// -- cCommands --------------------------------------------------------------
+// --- cCommands -------------------------------------------------------------
 
 cCommands Commands;
 cCommands RecordingCommands;
 
-// -- cSVDRPhosts ------------------------------------------------------------
+// --- cSVDRPhosts -----------------------------------------------------------
 
 cSVDRPhosts SVDRPhosts;
 
@@ -142,7 +141,7 @@ bool cSVDRPhosts::Acceptable(in_addr_t Address)
   return false;
 }
 
-// -- cSetupLine -------------------------------------------------------------
+// --- cSetupLine ------------------------------------------------------------
 
 cSetupLine::cSetupLine(void)
 {
@@ -208,7 +207,7 @@ bool cSetupLine::Save(FILE *f)
   return fprintf(f, "%s%s%s = %s\n", plugin ? plugin : "", plugin ? "." : "", name, value) > 0;
 }
 
-// -- cSetup -----------------------------------------------------------------
+// --- cSetup ----------------------------------------------------------------
 
 cSetup Setup;
 
@@ -284,7 +283,9 @@ cSetup::cSetup(void)
   FontSmlSize = 18;
   FontFixSize = 20;
   MaxVideoFileSize = MAXVIDEOFILESIZE;
+  MaxRecordingSize = DEFAULTRECORDINGSIZE;
   SplitEditedFiles = 0;
+  HardLinkCutter = 1;
   MinEventTimeout = 30;
   MinUserInactivity = 300;
   NextWakeupTime = 0;
@@ -297,6 +298,12 @@ cSetup::cSetup(void)
   InitialChannel = 0;
   InitialVolume = -1;
   EmergencyExit = 1;
+  LiveBuffer = false;
+  KeepPaused = false;
+  LiveBufferSize = 100;
+  InRAM = false;
+  MemBufSize = 5;
+  ExtendBuffer = false;
   ShowRecDate = 1;
   ShowRecTime = 1;
   ShowRecLength = 0;
@@ -338,10 +345,7 @@ void cSetup::Store(const char *Name, const char *Value, const char *Plugin, bool
 
 void cSetup::Store(const char *Name, int Value, const char *Plugin)
 {
-  char *buffer = NULL;
-  asprintf(&buffer, "%d", Value);
-  Store(Name, buffer, Plugin);
-  free(buffer);
+  Store(Name, cString::sprintf("%d", Value), Plugin);
 }
 
 bool cSetup::Load(const char *FileName)
@@ -475,7 +479,9 @@ bool cSetup::Parse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "FontSmlSize"))         FontSmlSize        = atoi(Value);
   else if (!strcasecmp(Name, "FontFixSize"))         FontFixSize        = atoi(Value);
   else if (!strcasecmp(Name, "MaxVideoFileSize"))    MaxVideoFileSize   = atoi(Value);
+  else if (!strcasecmp(Name, "MaxRecordingSize"))    MaxRecordingSize   = atoi(Value);
   else if (!strcasecmp(Name, "SplitEditedFiles"))    SplitEditedFiles   = atoi(Value);
+  else if (!strcasecmp(Name, "HardLinkCutter"))      HardLinkCutter     = atoi(Value);
   else if (!strcasecmp(Name, "MinEventTimeout"))     MinEventTimeout    = atoi(Value);
   else if (!strcasecmp(Name, "MinUserInactivity"))   MinUserInactivity  = atoi(Value);
   else if (!strcasecmp(Name, "NextWakeupTime"))      NextWakeupTime     = atoi(Value);
@@ -488,6 +494,12 @@ bool cSetup::Parse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "InitialChannel"))      InitialChannel     = atoi(Value);
   else if (!strcasecmp(Name, "InitialVolume"))       InitialVolume      = atoi(Value);
   else if (!strcasecmp(Name, "EmergencyExit"))       EmergencyExit      = atoi(Value);
+  else if (!strcasecmp(Name, "LiveBuffer"))          LiveBuffer         = atoi(Value);
+  else if (!strcasecmp(Name, "KeepPaused"))          KeepPaused         = atoi(Value);
+  else if (!strcasecmp(Name, "LiveBufferSize"))      LiveBufferSize     = atoi(Value);
+  else if (!strcasecmp(Name, "InRAM"))               InRAM              = atoi(Value);
+  else if (!strcasecmp(Name, "MemBufSize"))          MemBufSize         = atoi(Value);
+  else if (!strcasecmp(Name, "ExtendBuffer"))        ExtendBuffer       = atoi(Value);
   else if (!strcasecmp(Name, "ShowRecDate"))         ShowRecDate        = atoi(Value);
   else if (!strcasecmp(Name, "ShowRecTime"))         ShowRecTime        = atoi(Value);
   else if (!strcasecmp(Name, "ShowRecLength"))       ShowRecLength      = atoi(Value);
@@ -584,7 +596,9 @@ bool cSetup::Save(void)
   Store("FontSmlSize",        FontSmlSize);
   Store("FontFixSize",        FontFixSize);
   Store("MaxVideoFileSize",   MaxVideoFileSize);
+  Store("MaxRecordingSize",   MaxRecordingSize);
   Store("SplitEditedFiles",   SplitEditedFiles);
+  Store("HardLinkCutter",     HardLinkCutter);
   Store("MinEventTimeout",    MinEventTimeout);
   Store("MinUserInactivity",  MinUserInactivity);
   Store("NextWakeupTime",     NextWakeupTime);
@@ -597,6 +611,12 @@ bool cSetup::Save(void)
   Store("InitialChannel",     InitialChannel);
   Store("InitialVolume",      InitialVolume);
   Store("EmergencyExit",      EmergencyExit);
+  Store("LiveBuffer",         LiveBuffer);
+  Store("KeepPaused",         KeepPaused);
+  Store("LiveBufferSize",     LiveBufferSize);
+  Store("InRAM",              InRAM);
+  Store("MemBufSize",         MemBufSize);
+  Store("ExtendBuffer",       ExtendBuffer);
   Store("ShowRecDate",        ShowRecDate);
   Store("ShowRecTime",        ShowRecTime);
   Store("ShowRecLength",      ShowRecLength);
