@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 1.303 2007/11/25 13:45:48 kls Exp $
+ * $Id: config.h 1.310 2008/03/23 10:26:10 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,13 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "1.5.13"
-#define VDRVERSNUM   10513  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "1.6.0"
+#define VDRVERSNUM   10600  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "1.5.13"
-#define APIVERSNUM   10513  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "1.6.0"
+#define APIVERSNUM   10600  // Version * 10000 + Major * 100 + Minor
 
 #define PLUGINPARAMPATCHVERSNUM 1
 
@@ -38,9 +38,9 @@
 // plugins to work with newer versions of the core VDR as long as no
 // VDR header files have changed.
 
-#define MAINMENUHOOKSVERSNUM 1.0
+#define LIEMIKUUTIO  119
 
-#define LIEMIKUUTIO  117
+#define MAINMENUHOOKSVERSNUM 1.0
 
 #define MAXPRIORITY 99
 #define MAXLIFETIME 99
@@ -269,7 +269,9 @@ public:
   int FontSmlSize;
   int FontFixSize;
   int MaxVideoFileSize;
+  int MaxRecordingSize;
   int SplitEditedFiles;
+  int HardLinkCutter;
   int MinEventTimeout, MinUserInactivity;
   time_t NextWakeupTime;
   int MultiSpeedMode;
@@ -281,6 +283,12 @@ public:
   int InitialChannel;
   int InitialVolume;
   int EmergencyExit;
+  int LiveBuffer;
+  int KeepPaused;
+  int LiveBufferSize;
+  int InRAM;
+  int MemBufSize;
+  int ExtendBuffer;
   int ShowRecDate, ShowRecTime, ShowRecLength, ShowProgressBar, MenuCmdPosition;
   int CutTimePatchEnabled;
 //ML
