@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 1.148 2007/11/17 15:39:28 kls Exp $
+ * $Id: device.c 1.157 2008/03/09 10:03:34 kls Exp $
  */
 
 #include "device.h"
@@ -14,6 +14,7 @@
 #include "audio.h"
 #include "channels.h"
 #include "i18n.h"
+#include "livebuffer.h"
 #include "player.h"
 #include "receiver.h"
 #include "status.h"
@@ -221,6 +222,7 @@ int cDevice::nextCardIndex = 0;
 int cDevice::currentChannel = 1;
 cDevice *cDevice::device[MAXDEVICES] = { NULL };
 cDevice *cDevice::primaryDevice = NULL;
+cDevice *cDevice::avoidDevice = NULL;
 
 cDevice::cDevice(void)
 {
@@ -273,11 +275,6 @@ cDevice::~cDevice()
   DetachAllReceivers();
   delete liveSubtitle;
   delete dvbSubtitleConverter;
-  delete nitFilter;
-  delete sdtFilter;
-  delete patFilter;
-  delete eitFilter;
-  delete sectionHandler;
   delete pesAssembler;
 }
 
@@ -472,6 +469,8 @@ cDevice *cDevice::GetDevice(int Index)
 
 cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView)
 {
+  cDevice *AvoidDevice = avoidDevice;
+  avoidDevice = NULL;
   // Collect the current priorities of all CAM slots that can decrypt the channel:
   int NumCamSlots = CamSlots.Count();
   int SlotPriority[NumCamSlots];
@@ -505,6 +504,8 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
       if (NumUsableSlots && SlotPriority[j] > MAXPRIORITY)
          continue; // there is no CAM available in this slot
       for (int i = 0; i < numDevices; i++) {
+          if (device[i] == AvoidDevice)
+             continue; // this device shall be temporarily avoided
           if (Channel->Ca() && Channel->Ca() <= CA_DVB_MAX && Channel->Ca() != device[i]->CardIndex() + 1)
              continue; // a specific card was requested, but not this one
           if (NumUsableSlots && !CamSlots.Get(j)->Assign(device[i], true))
@@ -523,6 +524,7 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
              imp <<= 1; imp |= LiveView ? !device[i]->IsPrimaryDevice() || ndr : 0;                                  // prefer the primary device for live viewing if we don't need to detach existing receivers
              imp <<= 1; imp |= !device[i]->Receiving() && (device[i] != cTransferControl::ReceiverDevice() || device[i]->IsPrimaryDevice()) || ndr; // use receiving devices if we don't need to detach existing receivers, but avoid primary device in local transfer mode
              imp <<= 1; imp |= device[i]->Receiving();                                                               // avoid devices that are receiving
+             imp <<= 2; imp |= cLiveBufferManager::Impact(device[i], Channel, LiveView);
              imp <<= 1; imp |= device[i] == cTransferControl::ReceiverDevice();                                      // avoid the Transfer Mode receiver device
              imp <<= 8; imp |= min(max(device[i]->Priority() + MAXPRIORITY, 0), 0xFF);                               // use the device with the lowest priority (+MAXPRIORITY to assure that values -99..99 can be used)
              imp <<= 8; imp |= min(max((NumUsableSlots ? SlotPriority[j] : 0) + MAXPRIORITY, 0), 0xFF);              // use the CAM slot with the lowest priority (+MAXPRIORITY to assure that values -99..99 can be used)
@@ -786,6 +788,22 @@ void cDevice::StartSectionHandler(void)
      }
 }
 
+void cDevice::StopSectionHandler(void)
+{
+  if (sectionHandler) {
+     delete nitFilter;
+     delete sdtFilter;
+     delete patFilter;
+     delete eitFilter;
+     delete sectionHandler;
+     nitFilter = NULL;
+     sdtFilter = NULL;
+     patFilter = NULL;
+     eitFilter = NULL;
+     sectionHandler = NULL;
+     }
+}
+
 int cDevice::OpenFilter(u_short Pid, u_char Tid, u_char Mask)
 {
   return -1;
@@ -916,7 +934,7 @@ bool cDevice::SwitchChannel(int Direction)
 
 eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 {
-  if (LiveView) {
+  if (LiveView && !(Setup.LiveBuffer && dynamic_cast<cLivePlayer *>(player) != NULL)) {
      StopReplay();
      DELETENULL(liveSubtitle);
      DELETENULL(dvbSubtitleConverter);
@@ -926,6 +944,9 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 
   bool NeedsTransferMode = Device != this;
 
+  if (LiveView && Setup.LiveBuffer)
+     NeedsTransferMode = true;
+
   eSetChannelResult Result = scrOk;
 
 //ML
@@ -961,6 +982,9 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 //ML-End
         cStatus::MsgChannelSwitch(this, 0); // only report status if we are actually going to switch the channel
         if (Device->SetChannel(Channel, false) == scrOk) // calling SetChannel() directly, not SwitchChannel()!
+           if (Setup.LiveBuffer)
+              cLiveBufferManager::ChannelSwitch(Device,Channel);
+           else
            cControl::Launch(new cTransferControl(Device, Channel->GetChannelID(), Channel->Vpid(), Channel->Apids(), Channel->Dpids(), Channel->Spids()));
         else
            Result = scrNoTransfer;
@@ -1067,6 +1091,10 @@ void cDevice::SetAudioTrackDevice(eTrackType Type)
 {
 }
 
+void cDevice::SetSubtitleTrackDevice(eTrackType Type)
+{
+}
+
 bool cDevice::ToggleMute(void)
 {
   int OldVolume = volume;
@@ -1214,6 +1242,10 @@ bool cDevice::SetCurrentSubtitleTrack(eTrackType Type, bool Manual)
         DELETENULL(dvbSubtitleConverter);
         }
      DELETENULL(liveSubtitle);
+     if (player)
+        player->SetSubtitleTrack(currentSubtitleTrack, GetTrack(currentSubtitleTrack));
+     else
+        SetSubtitleTrackDevice(currentSubtitleTrack);
      if (currentSubtitleTrack != ttNone && !Replaying() && !Transferring()) {
         const tTrackId *TrackId = GetTrack(currentSubtitleTrack);
         if (TrackId && TrackId->id) {
@@ -1260,8 +1292,8 @@ void cDevice::EnsureAudioTrack(bool Force)
 void cDevice::EnsureSubtitleTrack(void)
 {
   if (Setup.DisplaySubtitles) {
-     eTrackType PreferredTrack = ttSubtitleFirst;
-     int LanguagePreference = -1;
+     eTrackType PreferredTrack = ttNone;
+     int LanguagePreference = INT_MAX; // higher than the maximum possible value
      for (int i = ttSubtitleFirst; i <= ttSubtitleLast; i++) {
          const tTrackId *TrackId = GetTrack(eTrackType(i));
          if (TrackId && TrackId->id && I18nIsPreferredLanguage(Setup.SubtitleLanguages, TrackId->language, LanguagePreference))
@@ -1328,7 +1360,7 @@ bool cDevice::Replaying(void) const
 
 bool cDevice::Transferring(void) const
 {
-  return dynamic_cast<cTransfer *>(player) != NULL;
+  return ActualDevice() != PrimaryDevice() || dynamic_cast<cLivePlayer *>(player) != NULL; // TODO
 }
 
 bool cDevice::AttachPlayer(cPlayer *Player)
@@ -1419,7 +1451,7 @@ int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
                break;
           case 0xC0 ... 0xDF: // audio
                SetAvailableTrack(ttAudio, c - 0xC0, c);
-               if (!VideoOnly && c == availableTracks[currentAudioTrack].id) {
+               if ((!VideoOnly || HasIBPTrickSpeed()) && c == availableTracks[currentAudioTrack].id) {
                   w = PlayAudio(Start, d, c);
                   if (FirstLoop)
                      Audios.PlayAudio(Data, Length, c);
@@ -1449,13 +1481,13 @@ pre_1_3_19_PrivateStreamDeteced:
                  case 0x20: // SPU
                  case 0x30: // SPU
                       SetAvailableTrack(ttSubtitle, SubStreamIndex, SubStreamId);
-                      if (!VideoOnly && currentSubtitleTrack != ttNone && SubStreamId == availableTracks[currentSubtitleTrack].id)
+                      if ((!VideoOnly || HasIBPTrickSpeed()) && currentSubtitleTrack != ttNone && SubStreamId == availableTracks[currentSubtitleTrack].id)
                          w = PlaySubtitle(Start, d);
                       break;
                  case 0x80: // AC3 & DTS
                       if (Setup.UseDolbyDigital) {
                          SetAvailableTrack(ttDolby, SubStreamIndex, SubStreamId);
-                         if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
+                         if ((!VideoOnly || HasIBPTrickSpeed()) && SubStreamId == availableTracks[currentAudioTrack].id) {
                             w = PlayAudio(Start, d, SubStreamId);
                             if (FirstLoop)
                                Audios.PlayAudio(Data, Length, SubStreamId);
@@ -1464,7 +1496,7 @@ pre_1_3_19_PrivateStreamDeteced:
                       break;
                  case 0xA0: // LPCM
                       SetAvailableTrack(ttAudio, SubStreamIndex, SubStreamId);
-                      if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
+                      if ((!VideoOnly || HasIBPTrickSpeed()) && SubStreamId == availableTracks[currentAudioTrack].id) {
                          w = PlayAudio(Start, d, SubStreamId);
                          if (FirstLoop)
                             Audios.PlayAudio(Data, Length, SubStreamId);
@@ -1580,7 +1612,7 @@ bool cDevice::Receiving(bool CheckAny) const
 
 #define TS_SCRAMBLING_CONTROL  0xC0
 #define TS_SCRAMBLING_TIMEOUT     3 // seconds to wait until a TS becomes unscrambled
-#define TS_SCRAMBLING_TIME_OK    10 // seconds before a Channel/CAM combination is marked a known to decrypt
+#define TS_SCRAMBLING_TIME_OK    10 // seconds before a Channel/CAM combination is marked as known to decrypt
 
 void cDevice::Action(void)
 {
