@@ -72,6 +72,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
          // not be overwritten.
          if (pEvent->TableID() == 0x00) {
             if (pEvent->Version() == getVersionNumber())
+               continue;
           {
           	if(Setup.MixEpgAction == 0)
           		continue;
@@ -173,7 +174,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
       if (newEvent)
          pSchedule->AddEvent(newEvent);
       if (Tid == 0x4E) { // we trust only the present/following info on the actual TS
-	 if (Setup.DisableVPS == 0 && SiEitEvent.getRunningStatus() >= SI::RunningStatusNotRunning)
+         if (Setup.DisableVPS == 0 && SiEitEvent.getRunningStatus() >= SI::RunningStatusNotRunning)
             pSchedule->SetRunningStatus(pEvent, SiEitEvent.getRunningStatus(), channel);
          }
       if (OnlyRunningStatus)
@@ -335,7 +336,6 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
       if (LinkChannels)
          channel->SetLinkChannels(LinkChannels);
       Modified = true;
-
       //to avoid double epg-entrys from ext and int epg sources :EW
       if (pEvent && pEvent->TableID() != 0x00)
       {
@@ -364,10 +364,10 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
        			int len_short_extern = 0;
        			
        			if (pEvent->ShortText())
-       				len_short_intern = sprintf (buffer_short_intern, "%s", pEvent->ShortText());
+       				len_short_intern = snprintf (buffer_short_intern, sizeof(buffer_short_intern), "%s", pEvent->ShortText());
        				
        			if (pPreviousEvent->ShortText())
-       				len_short_extern = sprintf (buffer_short_extern, "%s",pPreviousEvent->ShortText());
+       				len_short_extern = snprintf (buffer_short_extern, sizeof(buffer_short_extern), "%s",pPreviousEvent->ShortText());
  			
        			if(len_short_intern > 0)
        			{
@@ -385,10 +385,10 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
        				int len_title_extern = 0;
        				
        				if (pEvent->Description())
-       					len_title_intern = sprintf (buffer_title_intern, "%s", pEvent->Description());
+       					len_title_intern = snprintf (buffer_title_intern, sizeof(buffer_title_intern), "%s", pEvent->Description());
        					
        				if (pPreviousEvent->Description())
-       					len_title_extern = sprintf (buffer_title_extern, "%s", pPreviousEvent->Description());
+       					len_title_extern = snprintf (buffer_title_extern, sizeof(buffer_title_extern), "%s", pPreviousEvent->Description());
        				
        				if(len_title_intern > 0)
  	      			{
