@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menu.h 1.89 2007/08/26 14:35:57 kls Exp $
+ * $Id: menu.h 1.91 2008/02/10 16:01:53 kls Exp $
  */
 
 #ifndef __MENU_H
@@ -57,8 +57,6 @@ public:
 
 class cMenuMain : public cOsdMenu {
 private:
-  time_t lastDiskSpaceCheck;
-  int lastFreeMB;
   bool replaying;
   cOsdItem *stopReplayItem;
   cOsdItem *cancelEditingItem;
@@ -157,6 +155,7 @@ private:
   int level;
   int recordingsState;
   int helpKeys;
+  bool SetFreeDiskDisplay(bool Force = false);
   void SetHelpKeys(void);
   void Set(bool Refresh = false);
   bool Open(bool OpenSubMenus = false);
@@ -180,7 +179,7 @@ private:
   cTimer *timer;
   cRecorder *recorder;
   const cEvent *event;
-  char *instantId;
+  cString instantId;
   char *fileName;
   bool GetEvent(void);
 public:
@@ -237,7 +236,7 @@ private:
   bool ShowProgress(bool Initial);
   void MarkToggle(void);
   void MarkJump(bool Forward);
-  void MarkMove(bool Forward, int Frames = 1);
+  void MarkMove(bool Forward);
   void EditCut(void);
   void EditTest(void);
 public:
