@@ -11,7 +11,7 @@
  * The cRepacker family's code was originally written by Reinhard Nissl <rnissl@gmx.de>,
  * and adapted to the VDR coding style by Klaus.Schmidinger@cadsoft.de.
  *
- * $Id: remux.c 1.63 2007/11/18 14:45:28 kls Exp $
+ * $Id: remux.c 1.64 2007/11/25 13:56:03 kls Exp $
  */
 
 #include "remux.h"
@@ -783,34 +783,34 @@ bool cAudioRepacker::IsValidAudioHeader(uint32_t Header, bool Mpeg2, int *FrameS
 
   if (FrameSize || FrameDuration) {
      static int samplingFrequencies[2][4] = { // all values are specified in Hz
-       { 44100, 48000, 32000, -1 }, // MPEG 1
-       { 22050, 24000, 16000, -1 }  // MPEG 2
-       };
+        { 44100, 48000, 32000, -1 }, // MPEG 1
+	{ 22050, 24000, 16000, -1 }  // MPEG 2
+	};
 
      static int slots_per_frame[2][3] = {
-       { 12, 144, 144 }, // MPEG 1, Layer I, II, III
-       { 12, 144,  72 }  // MPEG 2, Layer I, II, III
-       };
+        { 12, 144, 144 }, // MPEG 1, Layer I, II, III
+	{ 12, 144,  72 }  // MPEG 2, Layer I, II, III
+	};
 
-     int mpegIndex = 1 - id;
-     int layerIndex = 3 - layer;
+        int mpegIndex = 1 - id;
+        int layerIndex = 3 - layer;
 
-     // Layer I (i. e., layerIndex == 0) has a larger slot size
-     int slotSize = (layerIndex == 0) ? 4 : 1; // bytes
-     int sf = samplingFrequencies[mpegIndex][sampling_frequency];
+	// Layer I (i. e., layerIndex == 0) has a larger slot size
+	int slotSize = (layerIndex == 0) ? 4 : 1; // bytes
+	int sf = samplingFrequencies[mpegIndex][sampling_frequency];
 
-     if (FrameDuration)
-        *FrameDuration = 90000 * 8 * slotSize * slots_per_frame[mpegIndex][layerIndex] / sf;
+	if (FrameDuration)
+           *FrameDuration = 90000 * 8 * slotSize * slots_per_frame[mpegIndex][layerIndex] / sf;
 
-     if (FrameSize) {
-        if (bitrate_index == 0)
-           *FrameSize = 0;
-        else {
-           int br = 1000 * bitRates[mpegIndex][layerIndex][bitrate_index]; // bits/s
-           int N = slots_per_frame[mpegIndex][layerIndex] * br / sf; // slots
+	if (FrameSize) {
+	   if (bitrate_index == 0)
+	      *FrameSize = 0;
+	   else {
+	      int br = 1000 * bitRates[mpegIndex][layerIndex][bitrate_index]; // bits/s
+	      int N = slots_per_frame[mpegIndex][layerIndex] * br / sf; // slots
 
-           *FrameSize = (N + padding_bit) * slotSize; // bytes
-           }
+              *FrameSize = (N + padding_bit) * slotSize; // bytes
+	   }
         }
      }
 
@@ -1774,12 +1774,12 @@ void cTS2PES::instant_repack(const uint8_t *Buf, int Count)
           case 7:
                   if (!done && (mpeg == 2 || mpeg1_required > 7)) {
                      flag2 = Buf[c++];
-		        if (!repacker && subStreamId && (flag2 & 0x01)) { //PES_EXTENSION=0x01
-			   esyslog("Error: cannot add extension to pes packet. Disabling.");
-			   subStreamId = 0;
-			   }
-			else
-			   flag2 |= 0x01; // PES_EXTENSION=0x01
+                     if (!repacker && subStreamId && (flag2 & 0x01)) { //PES_EXTENSION=0x01
+                        esyslog("Error: cannot add extension to pes packet. Disabling.");
+                        subStreamId = 0;
+                        }
+                     else
+                        flag2 |= 0x01; // PES_EXTENSION=0x01
                      found++;
                      }
                   break;
@@ -1925,29 +1925,6 @@ void cTS2PES::ts_to_pes(const uint8_t *Buf) // don't need count (=188)
      instant_repack(Buf + 4 + off, TS_SIZE - 4 - off);
 }
 
-// --- cRingBufferLinearPes --------------------------------------------------
-
-class cRingBufferLinearPes : public cRingBufferLinear {
-protected:
-  virtual int DataReady(const uchar *Data, int Count);
-public:
-  cRingBufferLinearPes(int Size, int Margin = 0, bool Statistics = false, const char *Description = NULL)
-  :cRingBufferLinear(Size, Margin, Statistics, Description) {}
-  };
-
-int cRingBufferLinearPes::DataReady(const uchar *Data, int Count)
-{
-  int c = cRingBufferLinear::DataReady(Data, Count);
-  if (!c && Count >= 6) {
-     if (!Data[0] && !Data[1] && Data[2] == 0x01) {
-        int Length = 6 + Data[4] * 256 + Data[5];
-        if (Length <= Count)
-           return Length;
-        }
-     }
-  return c;
-}
-
 // --- cAudioIndexer ---------------------------------------------------------
 
 class cAudioIndexer {
@@ -2000,6 +1977,29 @@ void cAudioIndexer::ProcessFrame(void)
   frameTrack = -1;
 }
 
+// --- cRingBufferLinearPes --------------------------------------------------
+
+class cRingBufferLinearPes : public cRingBufferLinear {
+protected:
+  virtual int DataReady(const uchar *Data, int Count);
+public:
+  cRingBufferLinearPes(int Size, int Margin = 0, bool Statistics = false, const char *Description = NULL)
+  :cRingBufferLinear(Size, Margin, Statistics, Description) {}
+  };
+
+int cRingBufferLinearPes::DataReady(const uchar *Data, int Count)
+{
+  int c = cRingBufferLinear::DataReady(Data, Count);
+  if (!c && Count >= 6) {
+     if (!Data[0] && !Data[1] && Data[2] == 0x01) {
+        int Length = 6 + Data[4] * 256 + Data[5];
+        if (Length <= Count)
+           return Length;
+        }
+     }
+  return c;
+}
+
 // --- cRemux ----------------------------------------------------------------
 
 #define RESULTBUFFERSIZE KILOBYTE(256)
@@ -2008,6 +2008,7 @@ cRemux::cRemux(int VPid, const int *APids, const int *DPids, const int *SPids, b
 {
   exitOnFailure = ExitOnFailure;
   noVideo = VPid == 0 || VPid == 1 || VPid == 0x1FFF;
+  audioIndexer = (noVideo ? new cAudioIndexer : NULL);
   numUPTerrors = 0;
   synced = false;
   syncEarly = SyncEarly;
@@ -2045,7 +2046,6 @@ cRemux::cRemux(int VPid, const int *APids, const int *DPids, const int *SPids, b
      while (*SPids && numTracks < MAXTRACKS && n < MAXSPIDS)
            ts2pes[numTracks++] = new cTS2PES(*SPids++, resultBuffer, IPACKS, 0x00, 0x20 + n++);
      }
-  audioIndexer = (noVideo ? new cAudioIndexer : NULL);
 }
 
 cRemux::~cRemux()
@@ -2221,8 +2221,10 @@ uchar *cRemux::Get(int &Count, uchar *PictureType)
                if (pt != NO_PICTURE) {
                   if (pt < I_FRAME || B_FRAME < pt) {
                      esyslog("ERROR: unknown picture type '%d'", pt);
-                     if (++numUPTerrors > MAXNUMUPTERRORS && exitOnFailure)
+                     if (++numUPTerrors > MAXNUMUPTERRORS && exitOnFailure) {
                         ShutdownHandler.RequestEmergencyExit();
+                        numUPTerrors = 0;
+                        }
                      }
                   else if (!synced) {
                      if (pt == I_FRAME || syncEarly) {
@@ -2235,8 +2237,8 @@ uchar *cRemux::Get(int &Count, uchar *PictureType)
                         if (syncEarly) {
                            if (pt == I_FRAME) // syncEarly: it's ok but there is no need to call SetBrokenLink()
                               SetBrokenLink(data + i, l);
-                           else fprintf(stderr, "video: synced early\n");
-                           }
+			    else fprintf(stderr, "video: synced early\n");
+			    }
                         }
                      }
                   else if (Count)
@@ -2249,21 +2251,21 @@ uchar *cRemux::Get(int &Count, uchar *PictureType)
                l = GetPacketLength(data, resultCount, i);
                if (l < 0)
                   return resultData;
-               if (noVideo || !synced && syncEarly) {
-                  uchar pt = NO_PICTURE;
-                  if (audioIndexer && !Count)
-                     audioIndexer->PrepareFrame(data, resultCount, i, pt); 
+	        if (noVideo || !synced && syncEarly) {
+	          uchar pt = NO_PICTURE;
+		  if (audioIndexer && !Count)
+		     audioIndexer->PrepareFrame(data, resultCount, i, pt); 
                   if (!synced) {
-                     if (PictureType && noVideo)
-                        *PictureType = pt;
+	              if (PictureType && noVideo)
+			*PictureType = pt;
                      resultSkipped = i; // will drop everything before this position
                      synced = true;
-                     if (!noVideo && syncEarly) fprintf(stderr, "audio: synced early\n");
+		      if (!noVideo && syncEarly) fprintf(stderr, "audio: synced early\n");
                      }
                   else if (Count)
                      return resultData;
                   else if (PictureType)
-                     *PictureType = pt;
+		     *PictureType = pt;
                   }
                }
             if (synced) {
